
;**************************************************************************
;*                    I2C Main Board                                      *
;**************************************************************************
;**************************************************************************
;*          v 2.57 - 26.10.2014                                           *
;*          I2C demo board                                                *
;*                                                                        *
;*          LCD & PS/2 keyboard routines included                         *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F887 used                                        *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************



;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;change history
;
;  v 2.17.2. - writes a hard defined calibration byte to RTC MCP7940N
;              the value is set to 000 (no change; values bellow 128 will speed up
;              the oscilator; values above 128 will slow down; values
;              0 and 128 are w/o influance
;  v 2.18    - test routines for SRAM I2C using PIC16F724
;  v 2.24    - clean-up: unsued code removed; all test codes removed:
;              for the SRAM-dev. / test routines refer to versions v2.18 - v.2.23
;  v 2.25    - EEPROM read routines implemented
;  v 2.27    - EEPROM edit functionality and write routines added
;  v 2.28    - negative values from themperature sensors can be displayed
;  v 2.29    - access to DS1624 EEPROM supported
;  v 2.30    - bug in EEPROM addressing for some types fixed
;  v 2.31    - adding new device type for EEPROM:
;                - parallel EEPROM AT28C256
;                - delay added after each byte send in write routine (for this type)
;  v 2.32    - test routines added for INA219 ("C"-key)
;  v 2.33    - voltage measurement with INA219 implemented
;  v 2.34    - main menu added (differentiation between "basics" and "special")
;  v 2.35    - first test of PCF8575 included
;  v 2.36    - "Ports" menu added
;  v 2.37    - binary editor included for Ports
;  v 2.38    - read operation for PCF8575 implemented
;  v 2.39    - PCF8575 implementation finished
;            - PCF8574 added
;  v 2.40    - PCA6107 added
;  v 2.41, 2.42
;            - NXP sub-menu added
;            - PCA9530 device added
;  v 2.43, 2.44
;            - PCA9531 device added
;  v 2.45    - PCA9532 device added - output only
;  v 2.46
;            - code clean-up; some unused parts removed
;            - special circuits routines moved to PAGE2 from PAGE3
;  v 2.47    - PCA9532 read routines included
;  v 2.48
;            - old communication routines for PCF8574 removed
;            - small adjustment of PCF8574 read included (send NOT ACK in the end of read cycle)
;  v 2.49    - first step in 1-wire included
;  v 2.50    - beta routines for BS18S20 1-wire temperature sensors implemented
;  v 2.51    - read ROM function for BS18S20 implemented
;            - temperature conversion for BS18S20 implemented
;  v 2.52    - structure of 1-wire menu adjsuted
;            - entry point changed to main menu (instead of "Basic Circuits" until v2.51)
;  v 2.53    - Ports menu structure adjusted
;  v 2.54    - code clean up
;  v 2.55    - TPIC2810 device added (Ports -> Group "X")
;  v 2.56    - support of PCA9534A included (Ports - Group "B" chip)
;  v 2.57    - support of MCP23008 included (Ports - Group "B" chip)
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;------------------------------------------------------
;device addresses:
;	temperature sensor:	1001 A2 A1 A0 x
;	PCF8574:		0100 A2 A1 A0 x
;	PCF857A:		0111 A2 A1 A0 x
;	PCF8575:		0100 A2 A1 A0 x
;	EEPROM:			1010 A2 A1 A0 x
;	RTC:			1101  0  0  0 x
;------------------------------------------------------



;	SDA -- SDA (RC4)
;	SCL -- SCL (RC3)

;keyboard
;	kbdclk  -- RB0
;	kbddata -- RB1


 INCLUDE "P16F887.INC"


_U001			     EQU     B'1110000011000100'	;see Page 206 of Datasheet
_U002			     EQU     B'1111111111111111'	;see Page 207 of Datasheet

;	__CONFIG	_CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_2_5 & _BOR_OFF & _CP_OFF & _MCLR_DIS & _PWRT_DIS & _WDT_OFF & _INTRC_OSC_NOCLKOUT
;	__CONFIG	_CONFIG2, _VCAP_DIS


	__CONFIG	_CONFIG1, _U001

	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

;interrupt
W_TEMP		EQU	H'22'
STATUS_TEMP	EQU	H'23'
PCLATH_TEMP	EQU	H'24'
data_tmp	EQU	H'25'
bus_D		EQU	H'26'
cnt1		EQU	H'27'
cnt2		EQU	H'28'
cnt3		EQU	H'29'
kbd_cnt1	EQU	H'2A'
kbd_buffer	EQU	H'2B'		;keyboard byte buffer
kbd_bb		EQU	H'2c'		;keyboard bit buffer
disp_pos	EQU	H'2D'
c_tmp		EQU	H'2E'

v_vt		EQU	H'2F'
v_tst1		EQU	H'30'		;temperature - char output 1
v_tst2		EQU	H'31'		;temperature - char output 2
v_tst3		EQU	H'32'		;temperature - char output 3
;---------------------------------------------------------------------
;---------------------------------------------------------------------
v_temp1s	EQU	H'33'
v_temp2s	EQU	H'34'


;---------------------------------------------------------------------
;EEPROM variables
v_ee_type	EQU	H'35'
	;selected EEPROM type:
	;	01 = 24LC01  (A:1010000; Add.bytes:1)		128 x 8		8
	;	02 = 24LC02  (A:1010000; Add.bytes:1)		256 x 8		8
	;	03 = 24LC04  (A:101000a; Add.bytes:1)		512 x 8		4
	;	04 = 24LC08  (A:10100aa; Add.bytes:1)		1024 x 8	2
	;	05 = 24LC16  (A:1010aaa; Add.bytes:1)		2048 x 8	1

	;	06 = 24LC32  (A:1010000; Add.bytes:2)		4096 x 8	8
	; 	07 = 24LC64  (A:1010000; Add.bytes:2)		8192 x 8	8
	;	08 = 24LC128 (A:1010000; Add.bytes:2)		16384 x 8	8
	;	09 = 24LC256 (A:1010000; Add.bytes:2)		32768 x 8	8
	;	0A = 24LC512 (A:1010000; Add.bytes:2)		65536 x 8	8

	;	0B = DS1624
	;	0C = AT28C256 connected via slave board with PIC16F724

;EEPROM communication buffer - 8 bytes
v_eed0		EQU	H'36'
v_eed1		EQU	H'37'
v_eed2		EQU	H'38'
v_eed3		EQU	H'39'
v_eed4		EQU	H'3A'
v_eed5		EQU	H'3B'
v_eed6		EQU	H'3C'
v_eed7		EQU	H'3D'

v_ee_add	EQU	H'3E'		;number of address bytes
v_ee_i2c	EQU	H'3F'		;EEPROM I2C address merged with memory address
v_ee_w		EQU	H'40'		;write mode active

v_b_in		EQU	H'41'		;register for data input from keyboard
b_ax_h		EQU	H'42'		;help register for data input
v_cursor	EQU	H'43'
v_comm		EQU	H'44'
;---------------------------------------------------------------------
;---------------------------------------------------------------------
;---------------------------------------------------------------------
v_i2c_add	EQU	H'45'		;I2C address

;variables 46 and 47 used bellow after 7F !!

;
pt_mode		EQU	H'48'		;bit0 = 1: M4 / bit1 = 1: L2
;
kaf		EQU	H'49'		;keyboard activity flag
;
vs_d0		EQU	H'4A'
vs_d1		EQU	H'4B'
vs_d2		EQU	H'4C'
vs_d3		EQU	H'4D'
;
kbd_out		EQU	H'4E'
kbd_parity	EQU	H'4F'

b_ax		EQU	H'50'
tmp_ot		EQU	H'51'

s_eeprom	EQU	H'52'

v_d0		EQU	H'53'		;EEPROM data - relative address 0
v_d1		EQU	H'54'		;EEPROM data - relative address 1
v_d2		EQU	H'55'		;EEPROM data - relative address 2
v_d3		EQU	H'56'		;EEPROM data - relative address 3
v_d4		EQU	H'57'		;EEPROM data - relative address 4
v_d5		EQU	H'58'		;EEPROM data - relative address 5

v_ad0		EQU	H'59'		;EEPROM address - lowest 8 bits
v_ad1		EQU	H'5A'		;EEPROM address - highest bits

v_byte		EQU	H'5B'		;data to be written to PCF8574
v_temp1		EQU	H'5C'		;temperature from TMP100 (MSB)
v_temp2		EQU	H'5D'		;temperature from TMP100 (LSB)

v_tempx		EQU	H'5E'		;temperature in dec
v_LED		EQU	H'5F'		;temperature for 8 LEDs

;RTC variables --------------------
v_rtc_d1	EQU	H'60'
v_rtc_d2	EQU	H'61'
v_rtc_m1	EQU	H'62'
v_rtc_m2	EQU	H'63'
v_rtc_y1	EQU	H'64'
v_rtc_y2	EQU	H'65'
v_rtc_y3	EQU	H'66'
v_rtc_y4	EQU	H'67'
v_rtc_h1	EQU	H'68'
v_rtc_h2	EQU	H'69'
v_rtc_n1	EQU	H'6A'
v_rtc_n2	EQU	H'6B'
v_rtc_s1	EQU	H'6C'
v_rtc_s2	EQU	H'6D'

v_rtc_sec	EQU	H'6E'
v_rtc_min	EQU	H'6F'
v_rtc_hrs	EQU	H'70'
v_rtc_wday	EQU	H'71'	;day in the week
v_rtc_day	EQU	H'72'
v_rtc_mnt	EQU	H'73'
v_rtc_year	EQU	H'74'

v_rtc_wd	EQU	H'75'
;---------------------------------
v_tmp1		EQU	H'76'	;temp variable - general usage

v_rtc_t1	EQU	H'77'
v_rtc_t2	EQU	H'78'

v_tmp2		EQU	H'79'

;--------------------------------------------------
;EEPROM communication buffer - old values - 8 bytes
o_eed0		EQU	H'7A'
o_eed1		EQU	H'7B'
o_eed2		EQU	H'7C'
o_eed3		EQU	H'7D'
o_eed4		EQU	H'7E'
o_eed5		EQU	H'7F'
o_eed6		EQU	H'46'
o_eed7		EQU	H'47'


;--------------------------------------------------
;-------------- registers BANK2 -------------------
;--------------------------------------------------
ow_ROM_0	EQU	H'110'
ow_ROM_1	EQU	H'111'
ow_ROM_2	EQU	H'112'
ow_ROM_3	EQU	H'113'
ow_ROM_4	EQU	H'114'
ow_ROM_5	EQU	H'115'
ow_ROM_6	EQU	H'116'
ow_ROM_7	EQU	H'117'


v_maxd1		EQU	H'133'
v_maxd2		EQU	H'134'

v_max_h1	EQU	H'135'
v_max_h2	EQU	H'136'
v_max_m1	EQU	H'137'
v_max_m2	EQU	H'138'
;
v_max_eds	EQU	H'139'		;"external LED display switch" - displaying clock or temperature
;
v_max_c1	EQU	H'13A'		;to be displayed on clk/tmp LED display
v_max_c2	EQU	H'13B'		;to be displayed on clk/tmp LED display
v_max_c3	EQU	H'13C'		;to be displayed on clk/tmp LED display
v_max_c4	EQU	H'13D'		;to be displayed on clk/tmp LED display
v_max_c5	EQU	H'13E'		;to be displayed on clk/tmp LED display
;
v_max_i		EQU	H'13F'
;
v_ina_msb	EQU	H'140'
v_ina_lsb	EQU	H'141'

v_ina_v1	EQU	H'142'
v_ina_v2	EQU	H'143'

v_volt_1	EQU	H'144'
v_volt_2	EQU	H'145'
v_volt_3	EQU	H'146'
v_volt_4	EQU	H'147'
v_volt_5	EQU	H'148'

v_t004		EQU	H'149'


v_p0		EQU	H'14A'
v_p1		EQU	H'14B'

v_cursor2	EQU	H'14C'

v_r0		EQU	H'14D'
v_r1		EQU	H'14E'

v_p_device	EQU	H'14F'		;device selected
;
v_pwm0		EQU	H'150'
v_pwm1		EQU	H'151'

v_psc0		EQU	H'152'
v_psc1		EQU	H'153'

v_ls0		EQU	H'154'
v_ls1		EQU	H'155'
v_ls2		EQU	H'156'
v_ls3		EQU	H'157'

v_pca953x	EQU	H'158'		;device type differentiation
v_F0		EQU	H'159'		;help variable
v_lsx		EQU	H'15A'		;help variable for LED status write routine

ds2484_dc	EQU	H'15B'
ds2484_pc	EQU	H'15C'
ds2484_st	EQU	H'15D'
ds2484_data	EQU	H'15E'

ow_buffer	EQU	H'15F'

ow_m1		EQU	H'160'
ow_m2		EQU	H'161'
ow_m3		EQU	H'162'
ow_m4		EQU	H'163'
ow_m5		EQU	H'164'
ow_m6		EQU	H'165'
ow_m7		EQU	H'166'
ow_m8		EQU	H'167'
ow_m9		EQU	H'168'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE -------------------------------------------------
;----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		ORG	004
		nop

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
PAGE0
		nop
		PAGESEL	PAGE2
		call	init_main
		PAGESEL	PAGE0
;
;-----------------------------------------------------------------------------
;init display - SCREEN1
;
		PAGESEL	PAGE1
		call	disp_init
		call	w_release
		PAGESEL	PAGE0
;
		call	rtc_v_init
;-------------------------------------------------------------------------------
;--- init phase is finished now ------------------------------------------------
;-------------------------------------------------------------------------------
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;
		PAGESEL	PAGE1
		call	clear_disp
		call	w_wkbd
		PAGESEL	PAGE0

;--- test ----
led_setup	movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000010'
		movwf	kbd_out
		call	send_ck
		bsf	PORTC,D'007'
		call	dr01

;		goto	main_escs

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Main entry screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m001		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_m001
		PAGESEL	PAGE0

m002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'32'		;B = Basic circuits
		btfsc	STATUS,Z
		goto	main_escs

		movf	kbd_buffer,0
		xorlw	H'1B'		;S = Special circuits
		btfsc	STATUS,Z
		goto	sic001_3

		goto	m002


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Special Circuits Screen Call
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sic001_3	nop
		PAGESEL	PAGE2
		goto	sic001
		PAGESEL	PAGE0

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Basic circuits screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
main_escs	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE0

main_es		nop
		PAGESEL	PAGE1
		call	w_i2c_s
		PAGESEL	PAGE0

mes_001		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = Real Time Clock
		btfsc	STATUS,Z
		goto	rtc_sc

		movf	kbd_buffer,0
		xorlw	H'2C'		;T = Temp sensorts
		btfsc	STATUS,Z
		goto	temp_m

		movf	kbd_buffer,0
		xorlw	H'24'		;E = EEPROM
		btfsc	STATUS,Z
		goto	eeprom_ssm

		movf	kbd_buffer,0
		xorlw	H'4D'		;P = Ports
		btfsc	STATUS,Z
		goto	ports1


		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	m001

		goto	mes_001


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                 PORTS - device first selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ports1		nop
		PAGESEL	PAGE3
		goto	ports1_3
		PAGESEL	PAGE0
		goto	main_escs


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          RTC - device selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_sc		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_rtcssc
		PAGESEL	PAGE0

scc001	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'		;1 = DS1307 / M41T00CAP
		btfsc	STATUS,Z
		goto	rtc_m1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2 = MCP7940N
		btfsc	STATUS,Z
		goto	rtc_m2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	main_escs

		goto	scc001


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          RTC main screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

rtc_m1		movlw	B'11010000'		;RTC = 1101000x
		movwf	v_i2c_add
		goto	rtc_m
;--------------------------------------------------------
rtc_m2		movlw	B'11011110'		;RTC = 1101111x
		movwf	v_i2c_add
		goto	rtc_m
;--------------------------------------------------------

rtc_m		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_rtcm
		PAGESEL	PAGE0

;----------------- RTC get data -----------------------
		movlw	H'F4'
		movwf	kbd_out
		call	send_ck
		call	d55

rtc_ml		call	rtc_i2c_read
		call	rtc_translate		;translate RTC date from RTC device to display format

		PAGESEL	PAGE1
		call	w_rtcm_dt
		PAGESEL	PAGE0
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

rwaitclk	btfss	PORTB,D'000'
		goto	rwaitclk

rwaitdat	btfss	PORTB,D'001'
		goto	rwaitdat

rwaitclk2	btfsc	PORTB,D'000'
		goto	rwaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'		;S - Set Date / time - keyboard input
		btfsc	STATUS,Z
		goto	rtc_set

		movf	kbd_buffer,0
		xorlw	H'24'		;E - Set Date / time example
		btfsc	STATUS,Z
		goto	rtc_set_e

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	rtemp_exit

		goto	rtc_ml

rtemp_exit	nop
		goto	rtc_sc


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;                          RTC set - example
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_set_e	nop

		call	rtc_i2c_write_e

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_rtc_e
		PAGESEL	PAGE0

rtcse01		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER
		btfsc	STATUS,Z
		goto	rtc_ml_p1

		goto	rtcse01


rtc_ml_p1	nop

		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_rtcm
		PAGESEL	PAGE0

		goto	rtc_ml
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;                          RTC set
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_set		nop
		PAGESEL	PAGE1
		call	disp_clr2
		call	w_rtcm_s
		PAGESEL	PAGE0
;

		movlw	B'10000000'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_d1

		movlw	B'10000001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_d2

		movlw	B'10000011'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_m1

		movlw	B'10000100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_m2

		movlw	B'10001000'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_y3

		movlw	B'10001001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_y4

		movlw	B'10001011'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_h1

		movlw	B'10001100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_h2

		movlw	B'10001110'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_n1

		movlw	B'10001111'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_n2

		movlw	B'10010001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_s1

		movlw	B'10010010'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_s2

		movlw	B'10010100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_wd

		goto	rtc_set		;start cyclic - from the first number again...

rtc_set_set	call	rtc_backtrs	;translate the info on the display to RTC device format
		call	rtc_i2c_write	;write data to RTC device
		goto	rtc_m

rtc_set_can	goto	rtc_m		;ESC - no changes to be made



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;               Temperature Sensor Selection Sreen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
temp_m		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_tm
		PAGESEL	PAGE0

tmm2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'		;1 = TMP100
		btfsc	STATUS,Z
		goto	tmp100c_m

		movf	kbd_buffer,0
		xorlw	H'1E'		;2 = AT30TSE754
		btfsc	STATUS,Z
		goto	tmp_atmel

		movf	kbd_buffer,0
		xorlw	H'26'		;3 = LM76
		btfsc	STATUS,Z
		goto	tmp_lm76

		movf	kbd_buffer,0
		xorlw	H'25'		;4 = DS1624
		btfsc	STATUS,Z
		goto	tmp_ds1624

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	main_escs

		goto	tmm2

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          TMP100c screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp100c_m	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_tmp100
		PAGESEL	PAGE0

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

tmp100c_rt	call	tmp100_read_t1
		call	t_convert2dec
		PAGESEL	PAGE1
		call	w_tmp100_t
		PAGESEL	PAGE0

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

waitclk		btfss	PORTB,D'000'
		goto	waitclk

waitdat		btfss	PORTB,D'001'
		goto	waitdat

waitclk2	btfsc	PORTB,D'000'
		goto	waitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp100c_rt

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          AT30TSE754 screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_atmel	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_tmp_atmel
		PAGESEL	PAGE0

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

tmp_atm_rt	call	tmp100_read_t2
		call	t_convert2dec
		PAGESEL	PAGE1
		call	w_tmp100_t
		PAGESEL	PAGE0

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

gwaitclk	btfss	PORTB,D'000'
		goto	gwaitclk

gwaitdat	btfss	PORTB,D'001'
		goto	gwaitdat

gwaitclk2	btfsc	PORTB,D'000'
		goto	gwaitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp_atm_rt


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          LM76 screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_lm76	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_tmp_lm76
		PAGESEL	PAGE0

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

tmp_ti_rt	call	tmp100_read_t2

		movf	v_temp1,0
		movwf	v_temp1s
		movf	v_temp2,0
		movwf	v_temp2s

		rlf	v_temp2,1
		rlf	v_temp1,1
		call	t_convert2dec

		movf	v_temp1s,0
		movwf	v_temp1
		movf	v_temp2s,0
		movwf	v_temp2

		PAGESEL	PAGE1
		call	w_tmp100_t
		PAGESEL	PAGE0

		call	dr2			;LM76 seems to be slow

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

twaitclk	btfss	PORTB,D'000'
		goto	twaitclk

twaitdat	btfss	PORTB,D'001'
		goto	twaitdat

twaitclk2	btfsc	PORTB,D'000'
		goto	twaitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp_ti_rt

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          DS1624 screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
tmp_ds1624	nop

		movlw	B'10010100'		;used I2C address for temperature sensors
		movwf	v_i2c_add

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_tmp_ds1624
		PAGESEL	PAGE0

		movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

		call	ds1624_init

tmp_mx_rt	call	tmp100_read_t3
		call	t_convert2dec
		PAGESEL	PAGE1
		call	w_tmp100_t
		PAGESEL	PAGE0

;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

mwaitclk	btfss	PORTB,D'000'
		goto	mwaitclk

mwaitdat	btfss	PORTB,D'001'
		goto	mwaitdat

mwaitclk2	btfsc	PORTB,D'000'
		goto	mwaitclk2

		clrf	kbd_buffer
		call	get_datafk
		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfsc	STATUS,Z
		goto	temp_m

		goto	tmp_mx_rt

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM selection SCREEN
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

eeprom_ssm	movlw	D'001'				;24LC01 EEPROM selected
		movwf	v_ee_type

eeprom_disp	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		call	w_eess
		PAGESEL	PAGE0

eeprom_ss	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = next EEPROM
		btfsc	STATUS,Z
		goto	eeprom_sel1

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = pervious EEPROM
		btfsc	STATUS,Z
		goto	eeprom_sel2

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eeprom_read

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eeprom_write

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	main_escs

		goto	eeprom_ss
;-----------------------------------------------------------------------------
eeprom_sel1	incf	v_ee_type,1
		movf	v_ee_type,0
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	eeprom_ssm

		goto	eeprom_disp
;-----------------------------------------------------------------------------
eeprom_sel2	decf	v_ee_type,1
		btfsc	STATUS,Z
		goto	eeprom_ssm2

		goto	eeprom_disp

eeprom_ssm2	movlw	H'0C'
		movwf	v_ee_type
		goto	eeprom_disp


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM write Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eeprom_write	bcf	PORTA,D'000'		;A0
		bcf	PORTA,D'001'		;A1
		bcf	PORTA,D'002'		;A2

		bcf	PORTC,D'005'		;WP = 0 (no write protect)


		clrf	v_ad0
		clrf	v_ad1


eep_lw2s	movlw	B'00000001'
		movwf	v_ee_w
		PAGESEL PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL PAGE0


eep_lw2		call	eep_i2c_read8		;EEPROM read 8 bytes from address v_ad1-v_ad0
		PAGESEL	PAGE1
		call	w_eeprom_data
		PAGESEL	PAGE0

;------------------------------------------------------------------
eep_rw1		movf	v_eed0,0
		movwf	v_b_in
		movlw	B'11000000'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed0
		call	eep_lw1

		movf	v_eed1,0
		movwf	v_b_in
		movlw	B'11000011'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed1
		call	eep_lw1

		movf	v_eed2,0
		movwf	v_b_in
		movlw	B'11000110'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed2
		call	eep_lw1

		movf	v_eed3,0
		movwf	v_b_in
		movlw	B'11001001'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed3
		call	eep_lw1

		movf	v_eed4,0
		movwf	v_b_in
		movlw	B'11001100'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed4
		call	eep_lw1

		movf	v_eed5,0
		movwf	v_b_in
		movlw	B'11001111'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed5
		call	eep_lw1

		movf	v_eed6,0
		movwf	v_b_in
		movlw	B'11010010'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed6
		call	eep_lw1

		movf	v_eed7,0
		movwf	v_b_in
		movlw	B'11010101'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE0
		movf	v_b_in,0
		movwf	v_eed7
		call	eep_lw1


		goto	eep_rw1
;------------------------------------------------------------------

eep_lw1		nop

;		clrf	kbd_buffer
;		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = read next
		btfsc	STATUS,Z
		goto	eep_lw5

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = read pervious
		btfsc	STATUS,Z
		goto	eep_lw6

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down = read next starting in next 256 bytes bank at 00
		btfsc	STATUS,Z
		goto	eep_lw3

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up = read next starting 256 bytes before at 00
		btfsc	STATUS,Z
		goto	eep_lw4

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eep_l2s

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	eeprom_disp

		return
;		goto	eep_lw1

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw3		call	ee_save

		clrf	v_ad0
		incf	v_ad1,1
		goto	eep_lw2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw4		call	ee_save

		clrf	v_ad0
		decf	v_ad1,1
		goto	eep_lw2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw5		call	ee_save

		call	eeprom_inc_ad8
		goto	eep_lw2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_lw6		call	ee_save

		call	eeprom_dec_ad8
		goto	eep_lw2
;-----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          EEPROM Read Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eeprom_read	bcf	PORTA,D'000'		;A0
		bcf	PORTA,D'001'		;A1
		bcf	PORTA,D'002'		;A2

		bcf	PORTC,D'005'		;WP = 0 (no write protect)

		clrf	v_ad0
		clrf	v_ad1

eep_l2s		clrf	v_ee_w
		PAGESEL PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL PAGE0


eep_l2		call	eep_i2c_read8		;EEPROM read 8 bytes from address v_ad1-v_ad0
		PAGESEL	PAGE1
		call	w_eeprom_data
		PAGESEL	PAGE0

eep_l1		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'74'		;-> = read next
		btfsc	STATUS,Z
		goto	eep_l5

		movf	kbd_buffer,0
		xorlw	H'6B'		;<- = read pervious
		btfsc	STATUS,Z
		goto	eep_l6

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down = read next starting in next 256 bytes bank at 00
		btfsc	STATUS,Z
		goto	eep_l3

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up = read next starting 256 bytes before at 00
		btfsc	STATUS,Z
		goto	eep_l4

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	eep_lw2s

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = back to EEPROM selection
		btfsc	STATUS,Z
		goto	eeprom_disp

		goto	eep_l1
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l3		clrf	v_ad0
		incf	v_ad1,1
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l4		clrf	v_ad0
		decf	v_ad1,1
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l5		call	eeprom_inc_ad8
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
eep_l6		call	eeprom_dec_ad8
		goto	eep_l2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

ee_save		nop

		movf	o_eed0,0
		xorwf	v_eed0,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed1,0
		xorwf	v_eed1,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed2,0
		xorwf	v_eed2,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed3,0
		xorwf	v_eed3,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed4,0
		xorwf	v_eed4,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed5,0
		xorwf	v_eed5,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed6,0
		xorwf	v_eed6,0
		btfss	STATUS,Z
		goto	ee_save_y

		movf	o_eed7,0
		xorwf	v_eed7,0
		btfss	STATUS,Z
		goto	ee_save_y

		return


ee_save_y	nop
		PAGESEL PAGE1
		call	disp_clr2
		call	w_ee_save
		PAGESEL PAGE0


		call	eep_i2c_write8
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;write 8 bytes to EEPROM
;   memory address in registers:	v_ad1  - v_ad0
;   data to EEPROM will be stored from: v_eed0 - v_eed7
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eep_i2c_write8	nop
;initialize the write operation
;
		call	i2c_start
;
		movlw	B'10100000'	;I2C address of the EEPROM device
		movwf	v_ee_i2c
;
;we have to merge the high address bits for EEPROMs 24LC04 - 24LC16 with the I2C address
;we will do it for 24LC01 and 02 as well to be able to work with then on different
;I2C addresses -- "finta"
		movf	v_ee_type,0
		xorlw	D'01'		;24LC01
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'02'		;24LC02
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'03'		;24LC04
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'04'		;24LC08
		btfsc	STATUS,Z
		goto	addr_merge2
		movf	v_ee_type,0
		xorlw	D'05'		;24LC16
		btfss	STATUS,Z
		goto	ee_i2c_cnt52x

addr_merge2	movf	v_ad1,0
		movwf	v_ee_i2c
		rlf	v_ee_i2c,0
		andlw	B'00001110'	;we are using only the 3 LSB bits for those devices
		movwf	v_ee_i2c

		movlw	B'10100000'	;I2C address of the EEPROM device
		addwf	v_ee_i2c,1
;----------------------------------------------------------------------
;now it is time to consider DS1624...
ee_i2c_cnt52x	movf	v_ee_type,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	ee_i2c_cnt52

;DS1624 - for now just change the device I2C address
		movlw	B'10010100'
		movwf	v_ee_i2c
;----------------------------------------------------------------------

;----------------------------------------------------------------------
ee_i2c_cnt52	movf	v_ee_i2c,0
		call	i2c_send


; -- DS1624 ? ----------------------------
		movf	v_ee_type,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		call	ee_DS1624_comm
; ----------------------------------------

		btfsc	v_ee_add,D'000'
		call	ee_sah
		movf	v_ad0,0
		call	i2c_send

;
;write the sequence of 8 bytes
		movf	v_eed0,0
		call	i2c_send
		call	fwpe

		movf	v_eed1,0
		call	i2c_send
		call	fwpe

		movf	v_eed2,0
		call	i2c_send
		call	fwpe

		movf	v_eed3,0
		call	i2c_send
		call	fwpe

		movf	v_eed4,0
		call	i2c_send
		call	fwpe

		movf	v_eed5,0
		call	i2c_send
		call	fwpe

		movf	v_eed6,0
		call	i2c_send
		call	fwpe

		movf	v_eed7,0
		call	i2c_send


		call	i2c_stop

		call	dr2

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
fwpe		movf	v_ee_type,0
		xorlw	H'0C'
		btfsc	STATUS,Z
		call	dr2			;wait a while for EEPROM AT28C512
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;read 8 bytes from EEPROM
;   memory address in registers:	v_ad1  - v_ad0
;   data from EEPROM will be stored in: v_eed0 - v_eed7
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
eep_i2c_read8	nop
;initialize the read operation
;
		call	i2c_start
;
		movlw	B'10100000'	;I2C address of the EEPROM device
		movwf	v_ee_i2c
;
;we have to merge the high address bits for EEPROMs 24LC04 - 24LC16 with the I2C address
;we will do it for 24LC01 and 02 as well to be able to work with then on different
;I2C addresses -- "finta"
		movf	v_ee_type,0
		xorlw	D'01'		;24LC01
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'02'		;24LC02
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'03'		;24LC04
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'04'		;24LC08
		btfsc	STATUS,Z
		goto	addr_merge
		movf	v_ee_type,0
		xorlw	D'05'		;24LC16
		btfss	STATUS,Z
		goto	ee_i2c_cnt5x

addr_merge	movf	v_ad1,0
		movwf	v_ee_i2c
		rlf	v_ee_i2c,0
		andlw	B'00001110'	;we are using only the 3 LSB bits for those devices
		movwf	v_ee_i2c

		movlw	B'10100000'	;I2C address of the EEPROM device
		addwf	v_ee_i2c,1
;----------------------------------------------------------------------
;now it is time to consider DS1624...
ee_i2c_cnt5x	movf	v_ee_type,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	ee_i2c_cnt5

;DS1624 - for now just change the device I2C address
		movlw	B'10010100'
		movwf	v_ee_i2c
;----------------------------------------------------------------------
ee_i2c_cnt5	movf	v_ee_i2c,0
		call	i2c_send

; -- DS1624 ? ----------------------------
		movf	v_ee_type,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		call	ee_DS1624_comm
; ----------------------------------------

		btfsc	v_ee_add,D'000'
		call	ee_sah		;send memory location address (MSB), if required
		movf	v_ad0,0		;memory location address (LSB)
		call	i2c_send

		call	i2c_start
		movf	v_ee_i2c,0	;I2C address of the EEPROM device
		addlw	D'001'		;set the LSB to 1 (--> read operation)
		call	i2c_send
;
;read the sequence of 8 bytes
		call	i2c_receive
		movwf	v_eed0
		movwf	o_eed0
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed1
		movwf	o_eed1
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed2
		movwf	o_eed2
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed3
		movwf	o_eed3
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed4
		movwf	o_eed4
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed5
		movwf	o_eed5
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed6
		movwf	o_eed6
		call	i2c_ack

		call	i2c_receive
		movwf	v_eed7
		movwf	o_eed7

; -- DS1624 ? ----------------------------
		movf	v_ee_type,0
		xorlw	H'0B'
		btfss	STATUS,Z
; ----------------------------------------
;send NOT ACK - for all EEPROM types except DS1624

		call	i2c_not_ack

		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;send high address byte
ee_sah		movf	v_ad1,0
		call	i2c_send
		return
;-----------------------------------------------------------------------------
;send EEPROM access command for DS1624 device
ee_DS1624_comm	movlw	H'17'
		call	i2c_send
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-----------------------------------------------------------------------------
;increment the address to be displayed
eeprom_inc_ad8	movlw	D'008'
		movwf	TIMER1
eaa1x		incf	v_ad0,1
		btfsc	STATUS,Z
		goto	inc_ah
cnt_lw		decfsz	TIMER1,1
		goto	eaa1x

		return
;-------------------------------------------
inc_ah		incf	v_ad1,1
		goto	cnt_lw
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;increment the address to be displayed
eeprom_dec_ad8	movlw	D'008'
		movwf	TIMER1
eaa1y		decf	v_ad0,1
		movf	v_ad0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	dec_ah
cnt_lw2		decfsz	TIMER1,1
		goto	eaa1y

		return
;-------------------------------------------
dec_ah		decf	v_ad1,1
		goto	cnt_lw2

;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
t_convert2dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3
;
		bsf	STATUS,RP1
		clrf	v_maxd1
		clrf	v_maxd2
		bcf	STATUS,RP1
;
		clrf	v_tempx
;
		movlw	D'255'
		xorwf	v_temp2,0		;if LSB is eq to 255 --- probably no sensor is connected
		btfsc	STATUS,Z
		goto	temp_no_value
;
		btfsc	v_temp1,D'007'		;negative value?
		goto	negative_temp
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;positive value conversion
;
		incf	v_temp1,0		;we convert only MSB now
		movwf	TIMER1			;store "to be converted" value to data location TIMER1
		
t_next		decfsz	TIMER1,1
		goto	t_inc_t3
		goto	finito7

t_inc_t3	call	t_inc_t
		goto	t_next
;we are done.....----
finito7		movf	v_tst1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	finito7_2
;
		movlw	' '		;use space instead of leading '0'
		movwf	v_tst1
;dblchk 2nd position as well
		movf	v_tst2,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	finito7_2
		movlw	' '		;use space instead of leading '0'
		movwf	v_tst2

finito7_2	movlw	'-'
		btfsc	v_temp1,D'007'
		movwf	v_tst1		;minus sign
		return
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;negative value conversion
;
negative_temp	movf	v_temp1,0
		movwf	TIMER1

nt_next		incf	TIMER1,1
		btfsc	STATUS,Z
		goto	finito9
		call	t_inc_t
		goto	nt_next
;------------------------------------
finito9		call	t_inc_t
		goto	finito7
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
t_inc_t		nop

		incf	v_tempx,1

		incf	v_tst3,1
;----------------------------------------
		bsf	STATUS,RP1
		incf	v_maxd2,1
		bCf	STATUS,RP1
;----------------------------------------
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	t_c_fin
;
		movlw	'0'
		movwf	v_tst3
;----------------------------------------
		bsf	STATUS,RP1
		clrf	v_maxd2
		incf	v_maxd1,1
		bcf	STATUS,RP1
;----------------------------------------
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	t_c_fin
;
		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

		return
;-----------------------------------
t_c_fin		nop		
		return

;-----------------------------------
temp_no_value	movlw	'-'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

		bsf	STATUS,RP1
		movlw	D'015'
		movwf	v_maxd1
		movwf	v_maxd2
		bcf	STATUS,RP1

		return
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;	RTC routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_translate	nop
		movf	v_rtc_sec,0
		xorlw	D'255'
		btfsc	STATUS,Z
		goto	rtc_v_init				;no RTC device connected

;day
		swapf	v_rtc_day,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_d1

		movf	v_rtc_day,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_d2


;month
		swapf	v_rtc_mnt,0
		andlw	B'00000001'
		addlw	'0'
		movwf	v_rtc_m1

		movf	v_rtc_mnt,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_m2

;year
		movlw	'2'
		movwf	v_rtc_y1
		movlw	'0'
		movwf	v_rtc_y2

		swapf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y3

		movf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y4

;hours
		swapf	v_rtc_hrs,0
		andlw	B'00000011'
		bsf	STATUS,RP1
		movwf	v_max_h1
		bcf	STATUS,RP1
		addlw	'0'
		movwf	v_rtc_h1

		movf	v_rtc_hrs,0
		andlw	B'00001111'
		bsf	STATUS,RP1
		movwf	v_max_h2
		bcf	STATUS,RP1
		addlw	'0'
		movwf	v_rtc_h2

;minutes
		swapf	v_rtc_min,0
		andlw	B'00000111'
		bsf	STATUS,RP1
		movwf	v_max_m1
		bcf	STATUS,RP1
		addlw	'0'
		movwf	v_rtc_n1

		movf	v_rtc_min,0
		andlw	B'00001111'
		bsf	STATUS,RP1
		movwf	v_max_m2
		bcf	STATUS,RP1
		addlw	'0'
		movwf	v_rtc_n2

;seconds
		swapf	v_rtc_sec,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_s1

		movf	v_rtc_sec,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_s2
;
;day in the week
		movf	v_rtc_wday,0
		andlw	B'00000111'
		movwf	v_rtc_wday

		movf	v_rtc_wday,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	rtc_set_Mo

		movf	v_rtc_wday,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	rtc_set_Tu

		movf	v_rtc_wday,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	rtc_set_We

		movf	v_rtc_wday,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	rtc_set_Th

		movf	v_rtc_wday,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	rtc_set_Fr

		movf	v_rtc_wday,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	rtc_set_Sa

		movf	v_rtc_wday,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	rtc_set_Su
;
		return

rtc_set_Mo	movlw	'M'
		movwf	v_rtc_t1
		movlw	'o'
		movwf	v_rtc_t2
		return

rtc_set_Tu	movlw	'T'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

rtc_set_We	movlw	'W'
		movwf	v_rtc_t1
		movlw	'e'
		movwf	v_rtc_t2
		return

rtc_set_Th	movlw	'T'
		movwf	v_rtc_t1
		movlw	'h'
		movwf	v_rtc_t2
		return

rtc_set_Fr	movlw	'F'
		movwf	v_rtc_t1
		movlw	'r'
		movwf	v_rtc_t2
		return

rtc_set_Sa	movlw	'S'
		movwf	v_rtc_t1
		movlw	'a'
		movwf	v_rtc_t2
		return

rtc_set_Su	movlw	'S'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

;-----------------------------------------------------------------------------
rtc_backtrs	nop
;date - day
		movlw	'0'
		subwf	v_rtc_d1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_d2,0
		addwf	v_tmp1,0

		movwf	v_rtc_day
;----------------------------------
;date - month
		movlw	'0'
		subwf	v_rtc_m1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_m2,0
		addwf	v_tmp1,0

		movwf	v_rtc_mnt
;----------------------------------
;date - year
		movlw	'0'
		subwf	v_rtc_y3,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_y4,0
		addwf	v_tmp1,0

		movwf	v_rtc_year
;----------------------------------
;date - day in the week - always use 1; and we will see...
		movlw	'0'
		subwf	v_rtc_wd,0
		movwf	v_rtc_wday

;----------------------------------
;time - hours
		movlw	'0'
		subwf	v_rtc_h1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_h2,0
		addwf	v_tmp1,0

		movwf	v_rtc_hrs
;----------------------------------
;time - minutes
		movlw	'0'
		subwf	v_rtc_n1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_n2,0
		addwf	v_tmp1,0

		movwf	v_rtc_min
;----------------------------------
;time - seconds
		movlw	'0'
		subwf	v_rtc_s1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_s2,0
		addwf	v_tmp1,0

		movwf	v_rtc_sec

		return
;-----------------------------------------------------------------------------
rtc_v_init	movlw	'-'
		movwf	v_rtc_d1
		movwf	v_rtc_d2
		movwf	v_rtc_m1
		movwf	v_rtc_m2
		movwf	v_rtc_y1
		movwf	v_rtc_y2
		movwf	v_rtc_y3
		movwf	v_rtc_y4
		movwf	v_rtc_h1
		movwf	v_rtc_h2
		movwf	v_rtc_n1
		movwf	v_rtc_n2
		movwf	v_rtc_s1
		movwf	v_rtc_s2

		movwf	v_rtc_t1
		movwf	v_rtc_t2

		return






;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
i2c_start	bsf	STATUS,RP0
		bsf	SSPCON2, SEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	STATUS,RP0
		bsf	SSPCON2, PEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	STATUS,RP0
		bcf	SSPCON2, ACKDT		;ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	STATUS,RP0
		bsf	SSPCON2, ACKDT		;NOT ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	STATUS,RP0
		bsf	SSPCON2, RCEN
		bcf	STATUS,RP0
		call	d55
		movf	SSPBUF,0
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	movwf	SSPBUF
		call	d55		;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	STATUS,RP0
		bsf	SSPCON2, RSEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------




;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read current date/time
;
rtc_i2c_read	nop

;Step 1 - Start condition
		call	i2c_start

;Step 2: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send

;Step 4: send STOP condition (stop the write)
		call	i2c_stop

;Step 5: START condition
		call	i2c_start

;Step 6: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

;-------------------------------------
;Step 7: read data
;7.1 - sec
		call	i2c_receive		;received byte - sec
		movwf	v_rtc_sec
		call	i2c_ack

;7.2 - min
		call	i2c_receive		;received byte - min
		movwf	v_rtc_min
		call	i2c_ack

;7.3 - hours
		call	i2c_receive		;received byte
		movwf	v_rtc_hrs
		call	i2c_ack

;7.4 - day in a week
		call	i2c_receive		;received byte
		movwf	v_rtc_wday
		call	i2c_ack

;7.5 - day
		call	i2c_receive		;received byte
		movwf	v_rtc_day
		call	i2c_ack

;7.6 - month
		call	i2c_receive		;received byte
		movwf	v_rtc_mnt
		call	i2c_ack

;7.7 - year
		call	i2c_receive		;received byte
		movwf	v_rtc_year
		call	i2c_not_ack

;Step 8: send STOP condition (stop the read)
		call	i2c_stop

		return

;--------------------------------------------------------------------------------------
;write example date / time to RTC: date = 15.11.2013 time = 14:32:45; day = 5 (Friday)
;--------------------------------------------------------------------------------------
rtc_i2c_write_e	nop

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send



;-------------------------------------
;Step 4: write data
;4.1 - sec
		movlw	B'01000101'		;BCD code of 45
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movlw	B'00110010'		;BCD code of 32
		call	i2c_send

;4.3 - hours
		movlw	B'00010100'		;BCD code of 14
		call	i2c_send

;4.4 - day in a week
		movlw	B'00000101'		;BCD code of 05
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movlw	B'00010101'		;BCD code of 15
		call	i2c_send

;4.6 - month
		movlw	B'00010001'		;BCD code of 11
		call	i2c_send

;4.7 - year
		movlw	B'00010011'		;BCD code of 13
		call	i2c_send

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_i2c_write	nop

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send
;-------------------------------------
;Step 4: write data
;4.1 - sec
		movf	v_rtc_sec,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movf	v_rtc_min,0
		call	i2c_send

;4.3 - hours
		movf	v_rtc_hrs,0
		call	i2c_send

;4.4 - day in a week
		movf	v_rtc_wday,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movf	v_rtc_day,0
		call	i2c_send

;4.6 - month
		movf	v_rtc_mnt,0
		call	i2c_send

;4.7 - year
		movf	v_rtc_year,0
		call	i2c_send

		btfsc	v_i2c_add,D'001'
		call	MCP7940_calib		;for MCP7940 - write calibration data in addition

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return

;-----------------------------------------------------------------------------
MCP7940_calib	movlw	H'80'
		call	i2c_send		;control register (just use the power-up value)

		movlw	D'000'
		call	i2c_send		;write 120 (DEC) to calibration register to speed-up the clock

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; TMP100 I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;---------------------------------------------------------------------------------
;read routine for temperature sensor TMP100, LM76 and AT30TSE754
;with pointer setting
;---------------------------------------------------------------------------------


tmp100_read_t1	nop

;1. write the address of required registerto the device
		call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	B'00000000'		;Register Addess = 00 => temperature register
		call	i2c_send

		call	i2c_stop

;2. read the temperature
		call	i2c_start

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

		return
;---------------------------------------------------------------------------------
;read routine for temperature sensor TMP100, LM76 and AT30TSE754
;w/o pointer setting
;---------------------------------------------------------------------------------
tmp100_read_t2	nop

;1. read the temperature
		call	i2c_start

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send		;addressing the device

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

		return
;---------------------------------------------------------------------------------
;read routine for temperature sensor DS1624
;---------------------------------------------------------------------------------

ds1624_init	call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'EE'			;command = start temperature conversion
		call	i2c_send

		call	i2c_stop
		return
;---------------------------------------------------
tmp100_read_t3	nop
;1. send command protocol
		call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	H'AA'			;command = read temeprature
		call	i2c_send

;2. read temeprature
		call	i2c_restart

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send		;addressing the device

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_temp2
		call	i2c_not_ack

		call	i2c_stop

		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sa_key		nop

		PAGESEL PAGE1
		call	w_rtcm_dt
		movf	tmp_ot,0
		call	w_otaznik
		PAGESEL PAGE0

sa_key3		btfss	PORTB,D'000'
		goto	sa_key2
		goto	sa_key3

;
sa_key2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC = return to menu
		btfsc	STATUS,Z
		goto	rtc_set_can	;"RTC set" cancel

		movf	kbd_buffer,0
		xorlw	H'24'		;E = Set
		btfsc	STATUS,Z
		goto	rtc_set_set	;RTC set the date and time

		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		goto	sa_key3

;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;-------------------------------------------------
hi_leave_esc	movlw	H'01'
		movwf	v_comm
		call	dr77
		return

hi_leave_enter	movlw	H'02'
		movwf	v_comm
		call	dr77
		return

hi_leave_F10	movlw	H'03'
		movwf	v_comm
		call	dr77
		return

hi_leave_oth	movlw	H'FE'
		movwf	v_comm
		call	dr77
		return
;-------------------------------------------------
;--------------------------------------------------------------------------
sa_key_a	movlw	H'00'
		movwf	v_comm

		PAGESEL PAGE1
		movf	tmp_ot,0
		call	send_comm
		PAGESEL PAGE0

sa_key3_a	btfss	PORTB,D'000'
		goto	sa_key2_a
		goto	sa_key3_a

;
sa_key2_a	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC
		btfsc	STATUS,Z
		goto	hi_leave_esc

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER
		btfsc	STATUS,Z
		goto	hi_leave_enter

		movf	kbd_buffer,0
		xorlw	H'09'		;F10
		btfsc	STATUS,Z
		goto	hi_leave_F10

		movf	kbd_buffer,0
		xorlw	H'74'		;->
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'6B'		;<-
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	hi_leave_oth




		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		movf	kbd_buffer,0
		xorlw	H'1C'		;A
		btfsc	STATUS,Z
		goto	set_avA

		movf	kbd_buffer,0
		xorlw	H'32'		;B
		btfsc	STATUS,Z
		goto	set_avB

		movf	kbd_buffer,0
		xorlw	H'21'		;C
		btfsc	STATUS,Z
		goto	set_avC

		movf	kbd_buffer,0
		xorlw	H'23'		;D
		btfsc	STATUS,Z
		goto	set_avD

		movf	kbd_buffer,0
		xorlw	H'24'		;E
		btfsc	STATUS,Z
		goto	set_avE

		movf	kbd_buffer,0
		xorlw	H'2B'		;F
		btfsc	STATUS,Z
		goto	set_avF

		goto	sa_key3_a
;-----------------------------------
set_av0		movlw	'0'
		movwf	b_ax
		movlw	H'00'
		movwf	b_ax_h
		call	dr77
		return

set_av1		movlw	'1'
		movwf	b_ax
		movlw	H'01'
		movwf	b_ax_h
		call	dr77
		return

set_av2		movlw	'2'
		movwf	b_ax
		movlw	H'02'
		movwf	b_ax_h
		call	dr77
		return

set_av3		movlw	'3'
		movwf	b_ax
		movlw	H'03'
		movwf	b_ax_h
		call	dr77
		return

set_av4		movlw	'4'
		movwf	b_ax
		movlw	H'04'
		movwf	b_ax_h
		call	dr77
		return

set_av5		movlw	'5'
		movwf	b_ax
		movlw	H'05'
		movwf	b_ax_h
		call	dr77
		return

set_av6		movlw	'6'
		movwf	b_ax
		movlw	H'06'
		movwf	b_ax_h
		call	dr77
		return

set_av7		movlw	'7'
		movwf	b_ax
		movlw	H'07'
		movwf	b_ax_h
		call	dr77
		return

set_av8		movlw	'8'
		movwf	b_ax
		movlw	H'08'
		movwf	b_ax_h
		call	dr77
		return

set_av9		movlw	'9'
		movwf	b_ax
		movlw	H'09'
		movwf	b_ax_h
		call	dr77
		return

set_avA		movlw	'A'
		movwf	b_ax
		movlw	H'0A'
		movwf	b_ax_h
		call	dr77
		return

set_avB		movlw	'B'
		movwf	b_ax
		movlw	H'0B'
		movwf	b_ax_h
		call	dr77
		return

set_avC		movlw	'C'
		movwf	b_ax
		movlw	H'0C'
		movwf	b_ax_h
		call	dr77
		return

set_avD		movlw	'D'
		movwf	b_ax
		movlw	H'0D'
		movwf	b_ax_h
		call	dr77
		return

set_avE		movlw	'E'
		movwf	b_ax
		movlw	H'0E'
		movwf	b_ax_h
		call	dr77
		return

set_avF		movlw	'F'
		movwf	b_ax
		movlw	H'0F'
		movwf	b_ax_h
		call	dr77
		return
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------
d7		MOVLW   D'127'
		MOVWF   TIMER2
DEL_LOOP71	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP72	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP72
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP71
		return

;------------------------------------

d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr2c		call	d2
		call	d2
		call	d2

		return
;------------------------------------
dr55		MOVLW   D'002'
		MOVWF   TIMER2
DEL_LOOPb1	MOVLW   D'100'
		MOVWF   TIMER1

DEL_LOOPb2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1
		return
;------------------------------------
dr56		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOPb1q	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPb2q	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2q
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1q
		return


;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;
dr55x		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		return
;
dr2n		call	d2
		call	d2
		return
;------------------------------------
dr77		call	d2
		return
;------------------------------------
;---------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;get a transmission from the keyboard
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

get_datafk	nop
		btfsc	PORTB,D'000'
		goto	kbd_err01		;clock level is 1 instead of expected 0
		btfsc	PORTB,D'001'
		goto	kbd_err02		;data level is 1 instead of expected 0 (start bit)
;
get_datafk2	nop
		clrf	kbd_buffer

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'000'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'001'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'002'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'003'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'004'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'005'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'006'

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		bsf	kbd_buffer,D'007'

;get parity bit

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		nop

;compare parity bit

		nop				;not now.... :-)

;wait for communication end - last bit should be "1"

		call	get_bitfk
		btfsc	kbd_bb,D'001'
		nop

		nop				;no check right now...

		return				;keyboard code is in kbd_buffer

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
write_char	nop
;send received data to PA
		movf	kbd_buffer,0
		movwf	bus_D
;		call	send_to_PA0
;and to the display
		movf	kbd_buffer,0
		call	convert2ascii

		call	d2

		return
;-----------------------------------------------------------------------------
;get bit from the keyboard during transmission
get_bitfk	nop
kbd_l2		btfss	PORTB,D'000'
		goto	kbd_l2			;wait to 1 on clock line
kbd_l3		btfsc	PORTB,D'000'
		goto	kbd_l3			;and now wait to 0 on clock
;
		movf	PORTB,0
		andlw	B'00000010'		;now we have the bit in W1
		movwf	kbd_bb

		return
;-----------------------------------------------------------------------------
kbd_err01	nop
		return
;-----------------------------
kbd_err02	nop
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; send data to the keyboard
;-----------------------------------------------------------------------------

send_ck		nop
		clrf	kbd_parity
		bsf	kbd_parity,D'000'

;-----------------------------------------------------------
;1.1. set RB4 (keaboard clock) to output ----------------------
		bsf	STATUS,RP0
		bcf	TRISB,D'000'
		bcf	STATUS,RP0
;-----------------------------------------------------------
;1.2. put KBD CLK to 0 ----------------------------------------
		bcf	PORTB,D'000'
		call	d57		;wait a while
;-----------------------------------------------------------
;2. put KBD DATA to 0 ---------------------------------------
		bsf	STATUS,RP0
		bcf	TRISB,D'001'
		bcf	STATUS,RP0

		bcf	PORTB,D'001'
;-----------------------------------------------------------
;3. release KBD CLK -----------------------------------------
		bsf	PORTB,D'000'

		bsf	STATUS,RP0
		bsf	TRISB,D'000'
		bcf	STATUS,RP0
;-----------------------------------------------------------
;4. wait for keayboard to bring the kbd clk low ------------
sck01		btfsc	PORTB,D'000'
		goto	sck01

		movf	kbd_out,0
		andlw	B'00000001'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000010'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000100'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00001000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00010000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00100000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'01000000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'10000000'
		call	send_kbd_bit

;now send the parity bit...
		movf	kbd_parity,0
		andlw	B'00000001'
		call	send_kbd_bit

;		movlw	B'00000001'		;stop bit
;		andlw	B'00000001'
;		call	send_kbd_bit

		bsf	STATUS,RP0
		bsf	TRISB,D'001'		;data as input
		bcf	STATUS,RP0

w0079		btfsc	PORTB,D'000'
		goto	w0079

w0080		btfss	PORTB,D'000'
		goto	w0080

w0081		btfsc	PORTB,D'000'
		goto	w0081

w0082		btfss	PORTB,D'000'
		goto	w0082

		return

;-----------------------------------------------------------
send_kbd_bit	btfsc	STATUS,Z
		goto	send_kbd0
		goto	send_kbd1

send_kbd0	bcf	PORTB,D'001'

w0077		btfss	PORTB,D'000'
		goto	w0077

w0078		btfsc	PORTB,D'000'
		goto	w0078

		return

send_kbd1	bsf	PORTB,D'001'
		movlw	B'00000001'
		xorwf	kbd_parity,1
		goto	w0077
;-----------------------------------------------------------------------------
d57		MOVLW   D'030'
		MOVWF   TIMER2
DEL_LOOP5721	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP5722	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP5722
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP5721
		return
;
dr01s		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN
;
;----------------------------------------------------------------------------
dr01		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP3_1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1
		RETURN
;-------------------------------------




;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIII      IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIII      III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; display routines --- PAGE 1 --------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


	ORG 800
PAGE1
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                       LCD Display Communication Routines                                     x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2
;R/W = RE1
;E   = RE0

;D0 - D3 -- not connected

;D4  = RD0
;D5  = RD1
;D6  = RD2
;D7  = RD3


disp_init	nop
;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTD
		clrf	PORTE
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTE,D'001'
		bcf	PORTE,D'002'
		bcf	PORTE,D'000'	;EN = 0 => disable


		movlw	B'00000011'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

		movlw	B'00000011'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

		movlw	B'00000011'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

		movlw	B'00000010'
		movwf	PORTD
		bsf	PORTE,D'000'
		call	dy_comm
		bcf	PORTE,D'000'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTE,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTD			;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTD			;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bcf	PORTE,D'001'		;R/W -> 0 (write)
		bsf	PORTE,D'002'		;RS -> 1 (data)
		bsf	PORTE,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTE,D'000'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTE,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTD			;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTD			;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTE,D'001'		;R/W -> 0 (write)
		bcf	PORTE,D'002'		;RS -> 0 (command)
		bsf	PORTE,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTE,D'000'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 255
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return
;-----------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
write_bin2	movf	v_ee_i2c,0
		movwf	kbd_buffer
		goto	write_bin7
;--------------------------------------------

w_i2c_add	call	wm_A
		call	wm_dvojbodka
		movf	v_i2c_add,0
		movwf	kbd_buffer

;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return
;---------------------------------------------------------------------------------


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; write texts to the display
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;---------------------------------------------------------------------------
w_pcsl		movlw	B'11001100'		;line 2 / char12
		call	send_comm

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		return
;---------------------------------------------------------------------------
w_otaznik	call	send_comm
;
		call	wm_otaznik
;
		movf	tmp_ot,0
		call	send_comm
		return
;---------------------------------------------------------------------------
w_scr5		movlw	B'10001100'		;line 1 / char12
		call	send_comm


		return
;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx
;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_I
		call	wm_2
		call	wm_C
		call	wm_space
		call	wm_D
		call	wm_e
		call	wm_m
		call	wm_o
		call	wm_space
		call	wm_b
		call	wm_o
		call	wm_a
		call	wm_r
		call	wm_d
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_v
		call	wm_2
		call	wm_point
		call	wm_5
		call	wm_7
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_6
		call	wm_point
		call	wm_1
		call	wm_0
		call	wm_point
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_4

		return
;-------------------------------------------------------------------------------
w_wkbd		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_w
		call	wm_a
		call	wm_i
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_f
		call	wm_o
		call	wm_r
		call	wm_space
		call	wm_k
		call	wm_e
		call	wm_y
		call	wm_b
		call	wm_o
		call	wm_a
		call	wm_r
		call	wm_d
		call	wm_point
		call	wm_point
		call	wm_point

		return
;-------------------------------------------------------------------------------
w_eess		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	'<'
		call	send_data
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_4
		call	wm_L
		call	wm_C

		movlw	B'10010110'		;row: 1 / col: 22
		call	send_comm

		call	wm_minus
		movlw	'>'
		call	send_data

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_A
		call	wm_dvojbodka
		call	wm_1
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_space
		call	wm_space
		call	wm_A
		call	wm_d
		call	wm_d
		call	wm_dvojbodka

		movlw	B'11010001'		;row: 2 / col: 17
		call	send_comm

		call	wm_sls
		call	wm_space
		call	wm_M
		call	wm_a
		call	wm_x
		call	wm_dvojbodka

		movlw	B'10000111'		;row: 1 / col: 7
		call	send_comm
		call	w_eeprom_type


		movlw	B'11011000'		;row: 2 / col: 24 (out of display)
		call	send_comm

		return
;-------------------------------------------------------------------------------
w_ee_save	movlw	B'11000000'
		call	send_comm
		call	wm_S
		call	wm_a
		call	wm_v
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_d
		call	wm_a
		call	wm_t
		call	wm_a
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;-------------------------------------------------------------------------------
w_eeprom_type	nop
		movf	v_ee_type,0
		xorlw	H'01'
		btfsc STATUS,Z
		goto	w_eeprom01

		movf	v_ee_type,0
		xorlw	H'02'
		btfsc STATUS,Z
		goto	w_eeprom02

		movf	v_ee_type,0
		xorlw	H'03'
		btfsc STATUS,Z
		goto	w_eeprom04

		movf	v_ee_type,0
		xorlw	H'04'
		btfsc STATUS,Z
		goto	w_eeprom08

		movf	v_ee_type,0
		xorlw	H'05'
		btfsc STATUS,Z
		goto	w_eeprom16

		movf	v_ee_type,0
		xorlw	H'06'
		btfsc STATUS,Z
		goto	w_eeprom32

		movf	v_ee_type,0
		xorlw	H'07'
		btfsc STATUS,Z
		goto	w_eeprom64

		movf	v_ee_type,0
		xorlw	H'08'
		btfsc STATUS,Z
		goto	w_eeprom128

		movf	v_ee_type,0
		xorlw	H'09'
		btfsc STATUS,Z
		goto	w_eeprom256

		movf	v_ee_type,0
		xorlw	H'0A'
		btfsc STATUS,Z
		goto	w_eeprom512

		movf	v_ee_type,0
		xorlw	H'0B'
		btfsc STATUS,Z
		goto	w_eepromDS

		movf	v_ee_type,0
		xorlw	H'0C'
		btfsc STATUS,Z
		goto	w_eepromp


w_eeprom_err	call	wm_e
		call	wm_r
		call	wm_r
		call	wm_o
		call	wm_r
		return

w_eeprom01	call	wm_0
		call	wm_1
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_1
		call	wm_2
		call	wm_8
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		clrf	v_ee_add		;one address byte

		return

w_eeprom02	call	wm_0
		call	wm_2
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_5
		call	wm_6
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		clrf	v_ee_add		;one address byte

		return

w_eeprom04	call	wm_0
		call	wm_4
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_5
		call	wm_1
		call	wm_2
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_4

		movlw	B'11001000'		;row: 2 / col: 8
		call	send_comm
		call	wm_x

		clrf	v_ee_add		;one address byte

		return

w_eeprom08	call	wm_0
		call	wm_8
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_1
		call	wm_0
		call	wm_2
		call	wm_4
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_2

		movlw	B'11000111'		;row: 2 / col: 7
		call	send_comm
		call	wm_x
		call	wm_x

		clrf	v_ee_add		;one address byte

		return

w_eeprom16	call	wm_1
		call	wm_6
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_0
		call	wm_4
		call	wm_8
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_1

		movlw	B'11000110'		;row: 2 / col: 6
		call	send_comm
		call	wm_x
		call	wm_x
		call	wm_x

		clrf	v_ee_add		;one address byte

		return

w_eeprom32	call	wm_3
		call	wm_2
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_4
		call	wm_0
		call	wm_9
		call	wm_6
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom64	call	wm_6
		call	wm_4
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_8
		call	wm_1
		call	wm_9
		call	wm_2
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom128	call	wm_1
		call	wm_2
		call	wm_8
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_1
		call	wm_6
		call	wm_3
		call	wm_8
		call	wm_4
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom256	call	wm_2
		call	wm_5
		call	wm_6
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_3
		call	wm_2
		call	wm_7
		call	wm_6
		call	wm_8
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return

w_eeprom512	call	wm_5
		call	wm_1
		call	wm_2
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_6
		call	wm_5
		call	wm_5
		call	wm_3
		call	wm_6
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_2

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return


w_eepromDS	movlw	B'10000011'		;row:1 / col: 3
		call	send_comm

		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_6
		call	wm_2
		call	wm_4
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_5
		call	wm_6
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_8

		movlw	B'11000010'		;row:2 / col: 2
		call	send_comm
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_1
		call	wm_0

		movlw	D'000'
		movwf	v_ee_add		;one address bytes

		return
;-------------------------------------------------------------------------------
;28C256

w_eepromp	movlw	B'10000011'		;row:1 / col: 3
		call	send_comm

		call	wm_2
		call	wm_8
		call	wm_C
		call	wm_2
		call	wm_5
		call	wm_6
		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_3
		call	wm_2
		call	wm_7
		call	wm_6
		call	wm_8
		call	wm_x
		call	wm_8

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	wm_1

		movlw	B'11010111'		;row: 2 / col: 23
		call	send_comm
		call	wm_minus

		movlw	B'11000010'		;row:2 / col: 2
		call	send_comm
		call	wm_1
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_0

		movlw	D'001'
		movwf	v_ee_add		;two address bytes

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_eeprom_data	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_A
		call	wm_d
		call	wm_d
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		movlw	':'
		btfsc	v_ee_w,D'000'
		movlw	'*'
		call	send_data
		call	wm_space

;first write the address (2 bytes)
		movf	v_ad1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus

		movf	v_ad0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		call	wm_A
		call	wm_dvojbodka
		call	write_bin2
;

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movf	v_eed0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed2,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed3,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed4,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed5,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed6,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	v_eed7,0
		movwf	kbd_buffer
		call	write_hex

;
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_i2c_s		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_R
		call	wm_zat2
		call	wm_T
		call	wm_C
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_T
		call	wm_zat2
		call	wm_e
		call	wm_m
		call	wm_p
		call	wm_e
		call	wm_r
		call	wm_a
		call	wm_t
		call	wm_u
		call	wm_r
		call	wm_e

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_E
		call	wm_P
		call	wm_R
		call	wm_O
		call	wm_M
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_P
		call	wm_zat2
		call	wm_o
		call	wm_r
		call	wm_t
		call	wm_s

		return
;-------------------------------------------------------------------------------
w_tmp100	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_T
		call	wm_M
		call	wm_P
		call	wm_hvizdicka
		call	wm_sls
		call	wm_A
		call	wm_T
		call	wm_hvizdicka
		call	wm_and
		call	wm_i
		call	wm_n
		call	wm_i
		call	wm_t
		goto	w_temp_all
;-------------------------------------------------------------------------------
w_tmp_atmel	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_T
		call	wm_M
		call	wm_P
		call	wm_hvizdicka
		call	wm_sls
		call	wm_A
		call	wm_T
		call	wm_hvizdicka
		call	wm_and
		call	wm_e
		call	wm_a
		call	wm_s
		call	wm_y
		goto	w_temp_all
;-------------------------------------------------------------------------------
w_tmp_lm76	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_L
		call	wm_M
		call	wm_7
		call	wm_6

		goto	w_temp_all
;-------------------------------------------------------------------------------
w_tmp_ds1624	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_6
		call	wm_2
		call	wm_4

		goto	w_temp_all
;-------------------------------------------------------------------------------
w_temp_all	movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		call	w_i2c_add

w_temp_t_all	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		call	wm_R
		call	wm_e
		call	wm_g
		call	wm_dvojbodka
		call	wm_space

		movlw	B'10001011'		;row: 1 / col: 11
		call	send_comm
		call	wm_minus
		call	wm_sip1
		call	wm_space
		call	wm_T
		call	wm_e
		call	wm_m
		call	wm_p
		call	wm_dvojbodka

		return
;-------------------------------------------------------------------------------
w_tmp100_t	movlw	B'10000101'		;row: 1 / col: 5
		call	send_comm

		movf	v_temp1,0		;MSB
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus

		movf	v_temp2,0		;LSB
		movwf	kbd_buffer
		call	write_hex

		movlw	B'10010011'		;row 1 / col: 19
		call	send_comm
		movf	v_tst1,0
		call	send_data
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		call	wm_stup
		call	wm_C

		return

;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_tm		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_1
		call	wm_zat2
		call	wm_sls
		call	wm_zat1
		call	wm_2
		call	wm_zat2
		call	wm_T
		call	wm_M
		call	wm_P
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_sls
		call	wm_A
		call	wm_T
		call	wm_3
		call	wm_0
		call	wm_T
		call	wm_S
		call	wm_E
		call	wm_7
		call	wm_5
		call	wm_4

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_zat1
		call	wm_3
		call	wm_zat2
		call	wm_L
		call	wm_M
		call	wm_7
		call	wm_6
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_zat1
		call	wm_4
		call	wm_zat2
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_6
		call	wm_2
		call	wm_4

		return
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_rtcssc	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_1
		call	wm_zat2
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_3
		call	wm_0
		call	wm_7
		call	wm_sls
		call	wm_M
		call	wm_4
		call	wm_1
		call	wm_T
		call	wm_0
		call	wm_0
		call	wm_C
		call	wm_A
		call	wm_P

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_2
		call	wm_zat2
		call	wm_M
		call	wm_C
		call	wm_P
		call	wm_7
		call	wm_9
		call	wm_4
		call	wm_0
		call	wm_N

		return		
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;RTC display date/time in first row
;
w_rtcm_dt	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movf	v_rtc_d1,0
		call	send_data
		movf	v_rtc_d2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_m1,0
		call	send_data
		movf	v_rtc_m2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_y1,0
		call	send_data
		movf	v_rtc_y2,0
		call	send_data
		movf	v_rtc_y3,0
		call	send_data
		movf	v_rtc_y4,0
		call	send_data

		call	wm_sls
		
		movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_s1,0
		call	send_data
		movf	v_rtc_s2,0
		call	send_data

		call	wm_space

		movf	v_rtc_t1,0
		call	send_data
		movf	v_rtc_t2,0
		call	send_data

		return
;-----------------------------------------------------------------
w_rtcm		call	w_rtcm_dt
;
;
		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_S
		call	wm_zat2
		call	wm_e
		call	wm_t
		call	wm_sls
		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_space
		call	wm_space
		call	w_i2c_add

		return
;-----------------------------------------------------------------
w_rtcm_s	call	w_rtcm_dt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_S
		call	wm_zat1
		call	wm_e
		call	wm_zat2
		call	wm_t
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C

		return
;-----------------------------------------------------------------
w_rtc_e		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_R
		call	wm_T
		call	wm_C
		call	wm_space
		call	wm_s
		call	wm_e
		call	wm_t
		call	wm_space
		call	wm_e
		call	wm_x
		call	wm_e
		call	wm_c
		call	wm_u
		call	wm_t
		call	wm_e
		call	wm_d
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		movlw	B'11000000'		;row: 1 / col: 0
		call	send_comm
		call	wm_P
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		call	wm_space
		call	wm_E
		call	wm_N
		call	wm_T
		call	wm_E
		call	wm_R
		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_m001		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_B
		call	wm_zat2
		call	wm_a
		call	wm_s
		call	wm_i
		call	wm_c
		call	wm_space
		call	wm_c
		call	wm_i
		call	wm_r
		call	wm_c
		call	wm_u
		call	wm_i
		call	wm_t
		call	wm_s

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_S
		call	wm_zat2
		call	wm_p
		call	wm_e
		call	wm_c
		call	wm_i
		call	wm_a
		call	wm_l
		call	wm_space
		call	wm_c
		call	wm_i
		call	wm_r
		call	wm_c
		call	wm_u
		call	wm_i
		call	wm_t
		call	wm_s

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------



;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIII           IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIII           III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII


	ORG	1000
PAGE2
		nop
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                          1-Wire Main Menu                                   X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ow_main_menu	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

		call	w_ow_mm

ow_mm		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'32'		;B = Basic Functions
		btfsc	STATUS,Z
		goto	ow_bafu

		movf	kbd_buffer,0
		xorlw	H'23'		;D = Device Selection
		btfsc	STATUS,Z
		goto	ow_dev_sel

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	sic001

		goto	ow_mm


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                       1-Wire Basic Functions                                X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ow_bafu		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

		call	w_ow_bafu

ow_bafu_mm	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'32'		;B = Bridge Registers (DS2484)
		btfsc	STATUS,Z
		goto	onew_s

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = Read ROM id
		btfsc	STATUS,Z
		goto	ow_rom_id

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_main_menu

		goto	ow_bafu_mm

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                       1-Wire Device Selection                               X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ow_dev_sel	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

		call	w_ow_dev_sel

ow_dsm		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'45'		;0 = DS18S20
		btfsc	STATUS,Z
		goto	ds1820_1

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to level above
		btfsc	STATUS,Z
		goto	ow_main_menu

		goto	ow_dsm


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                1-Wire Bridge Registers / DS2484                             X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
onew_s		nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

ds2484_r	call	ds2484_i2c_read
		call	w_ds2484

ds2484_m	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'22'		;X = re-read
		btfsc	STATUS,Z
		goto	ds2484_r

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_bafu

		goto	ds2484_m

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                            ROM id                                           X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

ow_rom_id	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
;------------------------------------------------------------
ow_rom_id_r	call	ds1820_read_rom
		call	w_ds1820i
;------------------------------------------------------------

ow_rom_id_im	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'22'		;X = re-read
		btfsc	STATUS,Z
		goto	ow_rom_id_r

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_bafu

		goto	ow_rom_id_im

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                   DS1820 - temperature measurement                          X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_1	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

;------------------------------------------------------------
ds1820_2	call	ds1820_read_all

		bsf	STATUS,RP1
		movf	ow_m1,0
		bcf	STATUS,RP1
		movwf	v_temp1
		bsf	STATUS,RP1
		movf	ow_m2,0
		bcf	STATUS,RP1
		movwf	v_temp2

		rrf	v_temp1,1
		bcf	v_temp1,D'007'
		PAGESEL	PAGE0
		call	t_convert2dec
		PAGESEL	PAGE2

		bsf	STATUS,RP1
		movf	ow_m1,0
		bcf	STATUS,RP1
		movwf	v_temp1
		bsf	STATUS,RP1
		movf	ow_m2,0
		bcf	STATUS,RP1
		movwf	v_temp2
;------------------------------------------------------------
		call	w_ds1820
;------------------------------------------------------------

ds1820_m	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'22'		;X = re-read temperature
		btfsc	STATUS,Z
		goto	ds1820_t1

		movf	kbd_buffer,0
		xorlw	H'21'		;C = re-read temperature with strong pull-up
		btfsc	STATUS,Z
		goto	ds1820_t2

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	ow_dev_sel

		goto	ds1820_m
;-------------------------------------------------------------------------------
ds1820_t1	nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

		call	ds1820_conv
		goto	ds1820_2
;-------------------------------------------------------------------------------
ds1820_t2	nop

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2

		call	ds1820_conv_spu
		goto	ds1820_2
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2484_i2c_read	call	ds2484_dc_read
		call	ds2484_pc_read
		call	ds2484_st_read
		call	ds2484_da_read
		return
;-------------------------------------------------------------------------------
ds2484_dc_read	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'C3'			;address of the Device Configuration Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	ds2484_dc
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
ds2484_pc_read	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'B4'			;address of the Port Configuration Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	ds2484_pc
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
ds2484_st_read	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'F0'			;address of the Status Register
		call	i2c_send
;
ds2484_st_read2	nop
		PAGESEL	PAGE0
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	ds2484_st
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
ds2484_da_read	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'E1'			;address of the Read Data Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	ds2484_data
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE2
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_conv	call	ow_rst
		bsf	STATUS,RP1
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write

		bsf	STATUS,RP1
		movlw	H'44'		;Convert Temperature
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write

		call	ow_rst

		return
;-------------------------------------------------------------------------------
ds1820_conv_spu	call	ow_rst
		bsf	STATUS,RP1
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write

		bsf	STATUS,RP1
		movlw	H'44'		;Convert Temperature
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write_spu

		call	d222		;wait a while...
		call	d222		;wait a while...
		call	d222		;wait a while...
		call	d222		;wait a while...
		call	d222		;wait a while...

		return
;-------------------------------------------------------------------------------
ds1820_read_all	call	ow_rst

		bsf	STATUS,RP1
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write

		bsf	STATUS,RP1
		movlw	H'BE'		;Read Scratchpad
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m1
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m2
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m3
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m4
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m5
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m6
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m7
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m8
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_m9
		bcf	STATUS,RP1

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_read_rom	call	ow_rst

		bsf	STATUS,RP1
		movlw	H'33'		;Read ROM
		movwf	ow_buffer
		bcf	STATUS,RP1
		call	ow_write

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_0
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_1
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_2
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_3
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_4
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_5
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_6
		bcf	STATUS,RP1

		call	ow_read
		bsf	STATUS,RP1
		movf	ow_buffer,0
		movwf	ow_ROM_7
		bcf	STATUS,RP1

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;###############################################################################
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;         1-wire communication routines utilizing bridge device DS2484
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;###############################################################################
;-------------------------------------------------------------------------------
;1-wire Reset
;-------------------------------------------------------------------------------
ow_rst		nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'B4'			;command: 1-wire Reset
		call	i2c_send

		call	i2c_stop

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
;1-wire write byte and use strong pull-up (SPU)
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write_spu	nop
		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'D2'			;command: Write Device Configuration
		call	i2c_send
		movlw	B'10110100'		;activate SPU bit of cinfiguration register
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		bsf	STATUS,RP1
		movf	ow_buffer,0
		bcf	STATUS,RP1
		call	i2c_send		;byte to be written to 1-wire bus

		call	i2c_stop

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
;1-wire write byte
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write	nop
		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		bsf	STATUS,RP1
		movf	ow_buffer,0
		bcf	STATUS,RP1
		call	i2c_send		;byte to be written to 1-wire bus

		call	i2c_stop

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
;1-wire read byte
;                   byte will be stored to: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_read		nop
		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'96'			;command: 1-wire Read Byte
		call	i2c_send
		call	i2c_stop

		PAGESEL	PAGE2
		call	ds2484_da_read

		bsf	STATUS,RP1
		movf	ds2484_data,0
		movwf	ow_buffer
		bcf	STATUS,RP1
		return
;-------------------------------------------------------------------------------
;###############################################################################
;###############################################################################
;###############################################################################

;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                            INA219 routines                                  X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ina219		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
;----------------------------------------------------------------------
		call	i2c_ina219_get		;get the value from INA219
		call	ina219_trans
;----------------------------------------------------------------------
		call	w_ina219_1
;----------------------------------------------------------------------
mi002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = Repeat
		btfsc	STATUS,Z
		goto	ina219

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfsc	STATUS,Z
		goto	sic001

		goto	mi002

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;translate received values into voltage information

ina219_trans	bsf	STATUS,RP1
		movf	v_ina_msb,0
		movwf	v_ina_v1
		movf	v_ina_lsb,0
		movwf	v_ina_v2

		rrf	v_ina_v1,1
		rrf	v_ina_v2,1
		rrf	v_ina_v1,1
		rrf	v_ina_v2,1
		rrf	v_ina_v1,1
		rrf	v_ina_v2,1
		movlw	B'00011111'
		andwf	v_ina_v1,1		;put the first three bits to '0'

		movlw	'0'
		movwf	v_volt_1
		movwf	v_volt_2
		movwf	v_volt_3
		movwf	v_volt_4
		movwf	v_volt_5


		incf	v_ina_v1,1
		movf	v_ina_v2,0
		btfsc	STATUS,Z
		goto	it56_5

		goto	it56_2

it56_1		movlw	D'000'
		movwf	v_ina_v2

it56_2		call	inc_inav_m		;increment V

		decfsz  v_ina_v2,F
		goto	it56_2
it56_5		decfsz  v_ina_v1,F
		goto	it56_1
;------------------------------------------------
;use space instead of very first '0' ------------
		movf	v_volt_1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	ina_t_ret
		movlw	' '
		movwf	v_volt_1
;------------------------------------------------
ina_t_ret	bcf	STATUS,RP1
		return
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
inc_inav_m	bsf	STATUS,RP1
		movlw	D'004'			;LSB = 4mV
		movwf	v_t004
;
inc_n3		incf	v_volt_5,1
		movlw	':'
		xorwf	v_volt_5,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_5
		incf	v_volt_4,1
		movlw	':'
		xorwf	v_volt_4,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_4
		incf	v_volt_3,1
		movlw	':'
		xorwf	v_volt_3,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_3
		incf	v_volt_2,1
		movlw	':'
		xorwf	v_volt_2,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_2
		incf	v_volt_1,1
		movlw	':'
		xorwf	v_volt_1,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'-'
		movwf	v_volt_1
		movwf	v_volt_2
		movwf	v_volt_3
		movwf	v_volt_4
		movwf	v_volt_5
		return

inc_n2		decfsz	v_t004,1
		goto	inc_n3

		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x               INA219  -  I2C communication routines                         x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
i2c_ina219_get	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'10000000'		;I2C address + write
		call	i2c_send
		movlw	D'002'			;pointer register
		call	i2c_send
		call	i2c_stop
;read
		call	i2c_start
		movlw	B'10000001'		;I2C address + read
		call	i2c_send
		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_ina_msb
		bcf	STATUS,RP1
		call	i2c_ack
;
		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_ina_lsb
		bcf	STATUS,RP1
		call	i2c_not_ack

		call	i2c_stop

		PAGESEL	PAGE2
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;X            Special Circuits Screens
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          1-Wire main screen                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ow_mm		movlw	B'10000000'
		PAGESEL	PAGE1
		call	send_comm
		call	wm_zat1
		call	wm_B
		call	wm_zat2
		call	wm_a
		call	wm_s
		call	wm_i
		call	wm_c
		call	wm_space
		call	wm_f
		call	wm_u
		call	wm_n
		call	wm_c
		call	wm_t
		call	wm_i
		call	wm_o
		call	wm_n
		call	wm_s

		movlw	B'11000000'
		call	send_comm
		call	wm_zat1
		call	wm_D
		call	wm_zat2
		call	wm_e
		call	wm_v
		call	wm_i
		call	wm_c
		call	wm_e
		call	wm_s

		PAGESEL	PAGE2
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          1-Wire Basic Functions                             x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ow_bafu	movlw	B'10000000'
		PAGESEL	PAGE1
		call	send_comm
		call	wm_zat1
		call	wm_B
		call	wm_zat2
		call	wm_r
		call	wm_i
		call	wm_d
		call	wm_g
		call	wm_e
		call	wm_space
		call	wm_r
		call	wm_e
		call	wm_g
		call	wm_i
		call	wm_s
		call	wm_t
		call	wm_e
		call	wm_r
		call	wm_s

		movlw	B'11000000'
		call	send_comm

		call	wm_zat1
		call	wm_R
		call	wm_zat2
		call	wm_O
		call	wm_M
		call	wm_space
		call	wm_i
		call	wm_d

		PAGESEL	PAGE2
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          1-Wire device selection                            x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ow_dev_sel	movlw	B'10000000'
		PAGESEL	PAGE1
		call	send_comm
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_S
		call	wm_2
		call	wm_zat1
		call	wm_0
		call	wm_zat2

		PAGESEL	PAGE2
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          DS1820  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ds1820i	movlw	B'10000000'
		PAGESEL	PAGE1
		call	send_comm
		call	wm_R
		call	wm_O
		call	wm_M
		call	wm_space
		call	wm_c
		call	wm_o
		call	wm_d
		call	wm_e
		call	wm_zat1
		call	wm_C
		call	wm_R
		call	wm_C
		call	wm_hviezdicka
		call	wm_S
		call	wm_N
		call	wm_minus
		call	wm_minus
		call	wm_F
		call	wm_a
		call	wm_m
		call	wm_zat2
		call	wm_dvojbodka

		movlw	B'11000000'
		call	send_comm

		bsf	STATUS,RP1
		movf	ow_ROM_7,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_hviezdicka

		bsf	STATUS,RP1
		movf	ow_ROM_6,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_ROM_5,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_ROM_4,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_ROM_3,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_ROM_2,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_ROM_1,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_ROM_0,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		PAGESEL	PAGE2
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ds1820	nop
		PAGESEL	PAGE1

		call	w_temp_t_all
		call	w_tmp100_t

		movlw	B'11000000'
		call	send_comm

		bsf	STATUS,RP1
		movf	ow_m3,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_m4,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_m5,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_m6,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_m7,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		bsf	STATUS,RP1
		movf	ow_m8,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_hviezdicka
		bsf	STATUS,RP1
		movf	ow_m9,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex

		call	wm_zat1
		call	wm_X
		call	wm_C
		call	wm_zat2

		PAGESEL	PAGE2
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          DS2484  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ds2484	movlw	B'10000000'
		PAGESEL	PAGE1
		call	send_comm
		call	wm_D
		call	wm_C
		call	wm_dvojbodka
		call	wm_space
		bsf	STATUS,RP1
		movf	ds2484_dc,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_S
		call	wm_t
		call	wm_a
		call	wm_t
		call	wm_dvojbodka
		call	wm_space

		bsf	STATUS,RP1
		movf	ds2484_st,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_dvojbodka
		call	wm_space
		bsf	STATUS,RP1
		movf	ds2484_pc,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_sls
		call	wm_space


		call	wm_D
		call	wm_a
		call	wm_t
		call	wm_a
		call	wm_dvojbodka
		call	wm_space

		bsf	STATUS,RP1
		movf	ds2484_data,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		movlw	B'11010100'
		call	send_comm
		call	wm_zat1
		call	wm_X
		call	wm_zat2

		PAGESEL	PAGE2
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          INA219  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ina219_1	movlw	B'10000000'		;row: 1 / col: 0
		PAGESEL	PAGE1
		call	send_comm

		call	wm_I
		call	wm_N
		call	wm_A
		call	wm_2
		call	wm_1
		call	wm_9
		call	wm_space
		call	wm_t
		call	wm_e
		call	wm_s
		call	wm_t

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_V
		call	wm_minus
		call	wm_r
		call	wm_e
		call	wm_g
		call	wm_dvojbodka
		call	wm_space

		bsf	STATUS,RP1
		movf	v_ina_msb,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_minus

		bsf	STATUS,RP1
		movf	v_ina_lsb,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_space
		call	wm_space

		bsf	STATUS,RP1
		movf	v_volt_1,0
		bcf	STATUS,RP1
		call	send_data

		bsf	STATUS,RP1
		movf	v_volt_2,0
		bcf	STATUS,RP1
		call	send_data

		movlw	','
		call	send_data

		bsf	STATUS,RP1
		movf	v_volt_3,0
		bcf	STATUS,RP1
		call	send_data

		bsf	STATUS,RP1
		movf	v_volt_4,0
		bcf	STATUS,RP1
		call	send_data

		bsf	STATUS,RP1
		movf	v_volt_5,0
		bcf	STATUS,RP1
		call	send_data

		call	wm_V

		PAGESEL	PAGE2
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_sic001	movlw	B'10000000'		;row: 1 / col: 0
		PAGESEL	PAGE1
		call	send_comm

		call	wm_zat1
		call	wm_I
		call	wm_zat2
		call	wm_N
		call	wm_A
		call	wm_2
		call	wm_1
		call	wm_9

		movlw	B'11000000'
		call	send_comm

		call	wm_zat1
		call	wm_1
		call	wm_zat2
		call	wm_minus
		call	wm_w
		call	wm_i
		call	wm_r
		call	wm_e

		PAGESEL	PAGE2
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Special Circuits Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sic001		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
		call	w_sic001

sic002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE2

		movf	kbd_buffer,0
		xorlw	H'43'		;I = INA219
		btfsc	STATUS,Z
		goto	ina219

		movf	kbd_buffer,0
		xorlw	H'16'		;1 = 1-wire Main Menu
		btfsc	STATUS,Z
		goto	ow_main_menu

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	sic002

		nop
		PAGESEL	PAGE0
		goto	m001

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

init_main	BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01011111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		bcf	OPTION_REG,T0CS	;set-up TIMER0 as timer
		bsf	OPTION_REG,PSA	;prescaler NOT for TIMER0
		;
		CLRF	INTCON		;Disable interrupts
		clrf	IOCB
		bsf	IOCB,D'000'	;enable interrupt on change for RB0 only

		MOVLW	B'11000000'	;PORTA
		MOVWF	TRISA

		MOVLW	B'11111111'	;RB0 = keyboard clock; RB1 = kbd data
		MOVWF	TRISB

		MOVLW	B'00011000'	;PORTC
		MOVWF	TRISC

		MOVLW	B'00110000'	;PORTD = input/output: RD0-RD3 - LCD; RD4-RD7 = PT2272-L2/A8-A9, D0-D1
		MOVWF	TRISD

		MOVLW	B'11111000'	;PORTE = output; RE3 = input
		MOVWF	TRISE

		movlw	B'00000011'
		movwf	WPUB

		bsf	STATUS,RP0
		bsf	STATUS,RP1
		clrf	ANSEL
		clrf	ANSELH

		bcf	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;set up I2C BUS for communication as a master
		movlw	B'00101000'		;see Datasheet Page 177
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSPCON

;set-up I2C speed
		bsf	STATUS,RP0
		movlw	B'00111111'
		movwf	SSPADD
		clrf	SSPSTAT
		bcf	STATUS,RP0
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------


; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       initialization -- Part 2     XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

init_main2	clrf	PORTC
		clrf	PORTA
		clrf	PORTB
;
		clrf	kaf

		clrf	PIE1
		clrf	PIE2

		bsf	INTCON,GIE
		bsf	INTCON,PEIE

		bsf	PIE1,SSPIE
;

		bcf	INTCON,GIE		;disable all interrupts
;----------------------------------------------------------------------------
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;in W is the command to be send to display (posintion in display)
;
;output will be shifted to the variable		'v_b_in'
;value of the variable				'v_comm' will be:
;
;							ESC		01
;							ENTER		02
;							F10		03
;							other		FF
;------------------------------------------------------------------------------
hex_input	movwf	v_cursor
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'11110000'
		andwf	b_ax_h,1
		rrf	b_ax_h,1
		rrf	b_ax_h,1
		rrf	b_ax_h,1
		rrf	b_ax_h,1

		PAGESEL	PAGE0
		call	sa_key_a
		PAGESEL	PAGE2
		movf	v_comm,0
		btfss	STATUS,Z
;exit - something happens
		goto	hi_exit_h

		movf	b_ax,0
		PAGESEL	PAGE1
		call	send_data
		PAGESEL	PAGE2
		call	hi_trans_h

		incf	v_cursor,0
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'00001111'
		andwf	b_ax_h,1

		PAGESEL	PAGE0
		call	sa_key_a
		PAGESEL	PAGE2
		movf	v_comm,0
		btfss	STATUS,Z
		goto	hi_exit_l

		movf	b_ax,0
		PAGESEL	PAGE1
		call	send_data
		PAGESEL	PAGE2
		call	hi_trans_l
		return

;now we have to translate the value back to HEX

hi_exit_h	call	hi_trans_h
		return

hi_exit_l	call	hi_trans_l
		return


hi_trans_h	rlf	b_ax_h,1
		rlf	b_ax_h,1
		rlf	b_ax_h,1
		rlf	b_ax_h,1
		movlw	B'11110000'
		andwf	b_ax_h,1
		movlw	B'00001111'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

hi_trans_l	movlw	B'00001111'
		andwf	b_ax_h,1
		movlw	B'11110000'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
d222		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP2221	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2222	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2222
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP2221
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;Some screens for Ports
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_demo8574	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_D
		call	wm_e
		call	wm_m
		call	wm_o
		call	wm_space
		call	wm_P
		call	wm_C
		call	wm_F
		call	wm_8
		call	wm_5
		call	wm_7
		call	wm_4

		PAGESEL	PAGE3
		goto	w_p2
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_ti	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_F
		call	wm_8
		call	wm_5
		call	wm_7
		call	wm_zat1
		call	wm_4
		call	wm_zat2
		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_P
		call	wm_C
		call	wm_F
		call	wm_8
		call	wm_5
		call	wm_7
		call	wm_zat1
		call	wm_5
		call	wm_zat2

		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_oth	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_T
		call	wm_P
		call	wm_I
		call	wm_zat1
		call	wm_C
		call	wm_zat2
		call	wm_2
		call	wm_8
		call	wm_1
		call	wm_0

		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_gb	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_zat1
		call	wm_6
		call	wm_zat2
		call	wm_1
		call	wm_0
		call	wm_7

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3
		call	wm_zat1
		call	wm_4
		call	wm_zat2
		call	wm_A

		movlw	B'11000000'
		call	send_comm

		call	wm_zat1
		call	wm_M
		call	wm_zat2
		call	wm_C
		call	wm_P
		call	wm_2
		call	wm_3
		call	wm_0
		call	wm_0
		call	wm_8

		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_nxp	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3
		call	wm_zat1
		call	wm_0
		call	wm_zat2

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3
		call	wm_zat1
		call	wm_1
		call	wm_zat2

		movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3
		call	wm_zat1
		call	wm_2
		call	wm_zat2


		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------


;-------------------------------------------------------------------------
w_pca6107	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'10000000'		;1/0
		call	send_comm
		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_6
		call	wm_1
		call	wm_0
		call	wm_7
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_0
		call	wm_0
		call	wm_1
		call	wm_1
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		PAGESEL	PAGE3
		goto	w_p0_main

;-------------------------------------------------------------------------
w_mcp23008	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'10000000'		;1/0
		call	send_comm
		call	wm_M
		call	wm_C
		call	wm_P
		call	wm_2
		call	wm_3
		call	wm_0
		call	wm_0
		call	wm_8
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		PAGESEL	PAGE3
		goto	w_p0_main

;-------------------------------------------------------------------------
w_pca9534A	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'10000000'		;1/0
		call	send_comm
		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3
		call	wm_4
		call	wm_A
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_0
		call	wm_1
		call	wm_1
		call	wm_1
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		PAGESEL	PAGE3
		goto	w_p0_main
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_pcf8574	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'10000000'		;1/0
		call	send_comm
		call	wm_P
		call	wm_C
		call	wm_F
		call	wm_8
		call	wm_5
		call	wm_7
		call	wm_4
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_0
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		PAGESEL	PAGE3
		goto	w_p0_main
;-------------------------------------------------------------------------
w_tpic2810	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'10000000'		;1/0
		call	send_comm
		call	wm_T
		call	wm_P
		call	wm_I
		call	wm_C
		call	wm_2
		call	wm_8
		call	wm_1
		call	wm_0
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_1
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		PAGESEL	PAGE3
		goto	w_p0_main

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ports_m_r	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_G
		call	wm_r
		call	wm_o
		call	wm_u
		call	wm_p
		call	wm_space
		call	wm_s
		call	wm_e
		call	wm_l
		call	wm_e
		call	wm_c
		call	wm_t
		call	wm_i
		call	wm_o
		call	wm_n
		call	wm_dvojbodka

		movlw	B'11000000'
		call	send_comm

		call	wm_zat1
		call	wm_A
		call	wm_zat2
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_B
		call	wm_zat2
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_C
		call	wm_zat2
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_X
		call	wm_zat2

		PAGESEL	PAGE3
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;MCP23008 communication routines
;  we are using BANK = 0 and autoincrement feature (power on defaults)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;GPIO as output
;-----------------------------------------------------------------------------
;configuration
;-----------------------------------------------------------------------------

p_send23008	bcf	STATUS,RP1

		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	B'00000000'		;address of the IODIR register
		call	i2c_send
		movlw	B'00000000'		;all bits of GPIO to output
		call	i2c_send
		call	i2c_stop
;-----------------------------------------------------------------------------
;send_data
;-----------------------------------------------------------------------------
		call	i2c_start
		movlw	B'01000000'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'09'			;GPIO register
		call	i2c_send
		bsf	STATUS,RP1
		movf	v_p0,0			;data to be send
		bcf	STATUS,RP1
		call	i2c_send
		call	i2c_stop

		PAGESEL	PAGE3
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;GPIO as input
;-----------------------------------------------------------------------------
;configuration
;-----------------------------------------------------------------------------

p_read23008	bcf	STATUS,RP1

		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	B'00000000'		;address of the IODIR register
		call	i2c_send
		movlw	B'11111111'		;all bits of GPIO to input
		call	i2c_send
		call	i2c_stop
;enable pull-ups
		call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'06'			;address of the GPPU register
		call	i2c_send
		movlw	B'11111111'		;enable all pull-up resistors
		call	i2c_send
		call	i2c_stop
;enable interrupt on change
		call	i2c_start
		movlw	B'01000000'		;MCP23008 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'02'			;address of the GPINTEN register
		call	i2c_send
		movlw	B'11111111'		;enable all pull-up resistors
		call	i2c_send
		call	i2c_stop
;-----------------------------------------------------------------------------
;read data
;-----------------------------------------------------------------------------
		call	i2c_start
		movlw	B'01000000'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'09'			;GPIO register
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movlw	B'01000001'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r0
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE3
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------



;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIII                IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIII  III  III  IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIII                IIII  IIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII



	ORG	1800

PAGE3
		nop

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                 PORTS - device first selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ports1_3	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3
		call	w_ports_m


p002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'1C'		;A = Quasi bi-directional GPIO
		btfsc	STATUS,Z
		goto	p_ti

		movf	kbd_buffer,0
		xorlw	H'32'		;B = Totem - Pole GPIO
		btfsc	STATUS,Z
		goto	p_g_b

		movf	kbd_buffer,0
		xorlw	H'21'		;C = LED drivers
		btfsc	STATUS,Z
		goto	p_nxp

		movf	kbd_buffer,0
		xorlw	H'22'		;X = other I/O devices
		btfsc	STATUS,Z
		goto	p_others

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z

		goto	p002

		nop
		PAGESEL	PAGE0
		goto	main_escs

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - LED drivers - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_nxp		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
		call	w_ports_nxp


p_nxp02		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'45'		;0 = PCA9530
		btfsc	STATUS,Z
		goto	p_pca9530_s

		movf	kbd_buffer,0
		xorlw	H'16'		;1 = PCA9531
		btfsc	STATUS,Z
		goto	p_pca9531_s

		movf	kbd_buffer,0
		xorlw	H'1E'		;2 = PCA9532
		btfsc	STATUS,Z
		goto	p_pca9532_s

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_nxp02
		goto	ports1_3

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 device selection
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pca9530_s	movlw	D'001'
		bsf	STATUS,RP1
		movwf	v_pca953x
		bcf	STATUS,RP1
		goto	p_pca9530_m
;----------------------------------------
p_pca9531_s	movlw	D'002'
		bsf	STATUS,RP1
		movwf	v_pca953x
		bcf	STATUS,RP1
		goto	p_pca9530_m
;----------------------------------------
p_pca9532_s	movlw	D'003'
		bsf	STATUS,RP1
		movwf	v_pca953x
		bcf	STATUS,RP1
		goto	p_pca9530_m
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 main screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pca9530_m	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3
		call	w_pca9530m

pca9530_02	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = PCA9530 - read
		btfsc	STATUS,Z
		goto	pca9530_r

		movf	kbd_buffer,0
		xorlw	H'4D'		;P = PCA9530 - PSC0/1 + PWM0/1 set
		btfsc	STATUS,Z
		goto	pca9530_pwm

		movf	kbd_buffer,0
		xorlw	H'4B'		;L = LEDs - register LS0
		btfsc	STATUS,Z
		goto	pca9530_L0

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	pca9530_02
		goto	p_nxp

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 PWM maintenance screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pca9530_pwm	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

pwm_33		call	p_pxx_i2c_read
		call	w_ppwm
;-------------------------------------------------------------
;PSC0
		bsf	STATUS,RP1
		movf	v_psc0,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'10000111'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_psc0
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;-------------------------------------------------------------
;PWM0
		bsf	STATUS,RP1
		movf	v_pwm0,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'10010101'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_pwm0
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;-------------------------------------------------------------
;PSC1
		bsf	STATUS,RP1
		movf	v_psc1,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'11000111'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_psc1
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;--------------------------------------------------------------
;PWM1
		bsf	STATUS,RP1
		movf	v_pwm1,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'11010101'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_pwm1
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		call	p_pxx_i2c_write
;-------------------------------------------------------------
		goto	pwm_33

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 read screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pca9530_r	nop
		PAGESEL	PAGE1
		call	disp_clr2
		PAGESEL	PAGE3

;--- read operation & display.....
		call	p9530_i2c_read
		call	w_pca9530r

pca9530r_02	clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = PCA9530 - read
		btfsc	STATUS,Z
		goto	pca9530_r

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	pca9530r_02
		goto	p_pca9530_m

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; NXP PCA9530 LED0/1 screen
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
pca9530_L0	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3
n_nxp_l1	call	p_pxx_i2c_read
		call	w_LED0

;------ LS0 ------------------------------------
		bsf	STATUS,RP1
		movf	v_ls0,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'10000110'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_ls0
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x998			;leave now for PCA9530 device

		bcf	STATUS,RP1
		call	p_pxx_i2c_write

		call	w_LED0

;------ LS1 ------------------------------------
		bsf	STATUS,RP1
		movf	v_ls1,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'10001010'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_ls1
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		bcf	STATUS,RP1
		call	p_pxx_i2c_write

		call	w_LED0

;------ LS2 ------------------------------------
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x998			;leave now for PCA9531 device
		bcf	STATUS,RP1

		bsf	STATUS,RP1
		movf	v_ls2,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'10001110'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_ls2
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC

		bcf	STATUS,RP1
		call	p_pxx_i2c_write

		call	w_LED0
;------ LS3 ------------------------------------
		bsf	STATUS,RP1
		movf	v_ls3,0
		bcf	STATUS,RP1
		movwf	v_b_in
		movlw	B'10010010'
		PAGESEL	PAGE2
		call	hex_input
		PAGESEL	PAGE3
		movf	v_b_in,0
		bsf	STATUS,RP1
		movwf	v_ls3
		bcf	STATUS,RP1

		movf	v_comm,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	p_pca9530_m		;ESC


x998		bcf	STATUS,RP1
		call	p_pxx_i2c_write

		goto	n_nxp_l1

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - Group X - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_others	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
		call	w_ports_oth

p_ot002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'21'		;C = TPIC2810
		btfsc	STATUS,Z
		goto	p_tpic2810

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_ot002
		goto	ports1_3


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - Group B - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_g_b		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
		call	w_ports_gb


p_gb002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'36'		;6 = PCA6107
		btfsc	STATUS,Z
		goto	p_pca6107

		movf	kbd_buffer,0
		xorlw	H'25'		;4 = PCA9534A
		btfsc	STATUS,Z
		goto	p_pca9534A

		movf	kbd_buffer,0
		xorlw	H'3A'		;M = MCP23008
		btfsc	STATUS,Z
		goto	p_mcp23008

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_gb002
		goto	ports1_3

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;          PORTS - Group A - selection screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

p_ti		nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE2
		call	w_ports_ti


p_ti002		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'25'		;4 = PCF8574
		btfsc	STATUS,Z
		goto	p_pcf8574

		movf	kbd_buffer,0
		xorlw	H'2E'		;5 = PCF8575
		btfsc	STATUS,Z
		goto	p_pcf8575

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC - back to start menu
		btfss	STATUS,Z
		goto	p_ti002
		goto	ports1_3
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_tpic2810	nop
		bsf	STATUS,RP1

		movlw	D'010'			;device = TPIC2810
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		bcf	STATUS,RP1

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

		goto	p_75_1
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pca6107	nop
		bsf	STATUS,RP1

		movlw	D'003'			;device = PCA6107
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		bcf	STATUS,RP1

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

		goto	p_75_1
;-----------------------------------------------------------------------------
p_pca9534A	bsf	STATUS,RP1
		movlw	D'004'			;device = PCA9534A
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2
		bcf	STATUS,RP1

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

		goto	p_75_1

;-----------------------------------------------------------------------------
p_mcp23008	nop
		bsf	STATUS,RP1

		movlw	D'005'			;device = MCP23008
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2
		bcf	STATUS,RP1

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

		goto	p_75_1

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pcf8574	nop
		bsf	STATUS,RP1

		movlw	D'001'			;device = PCF8574
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0

		movlw	D'007'
		movwf	v_cursor2

		goto	p_75_3

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
p_pcf8575	nop
		bsf	STATUS,RP1

		movlw	D'002'			;device = PCF8575
		movwf	v_p_device

		movlw	B'11111111'
		movwf	v_p0
		movwf	v_p1

		movlw	D'015'
		movwf	v_cursor2
p_75_3		bcf	STATUS,RP1

		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

		call	p_send1		;set all ports to "1"
;
		call	p_read1		;read the port values

;------------------------------------------------------
p_75_1		call	w_pcf8575
		bcf	STATUS,RP1

p_75_2		clrf	kbd_buffer
		PAGESEL	PAGE0
		call	get_datafk
		PAGESEL	PAGE3

		movf	kbd_buffer,0
		xorlw	H'1B'		;S - Send data to the device
		btfsc	STATUS,Z
		call	p_send1

		movf	kbd_buffer,0
		xorlw	H'2D'		;R - Read data from the device
		btfsc	STATUS,Z
		goto	p_read2

		movf	kbd_buffer,0
		xorlw	H'23'		;D - Demo
		btfsc	STATUS,Z
		goto	p_demo7

		movf	kbd_buffer,0
		xorlw	H'45'		;0 - 0 was pressed
		btfsc	STATUS,Z
		goto	p_key0

		movf	kbd_buffer,0
		xorlw	H'16'		;1 - 1 was pressed
		btfsc	STATUS,Z
		goto	p_key1

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC -> exit
		btfss	STATUS,Z
		goto	p_75_2

;------------------------------------------------------
;return procedure
;------------------------------------------------------
		bsf	STATUS,RP1
		movf	v_p_device,0
		bcf	STATUS,RP1
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	p_others		;return to "Others" sub-menu

		bsf	STATUS,RP1
		movf	v_p_device,0
		bcf	STATUS,RP1
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	p_g_b

		bsf	STATUS,RP1
		movf	v_p_device,0
		bcf	STATUS,RP1
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	p_g_b

		bsf	STATUS,RP1
		movf	v_p_device,0
		bcf	STATUS,RP1
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	p_g_b


		goto	p_ti

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read2		call	p_read1
		goto	p_75_1
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_demo7		bsf	STATUS,RP1
		movf	v_p_device,0
		bcf	STATUS,RP1
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	p_demo8574

		goto	p_75_2
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_demo8574	nop
		PAGESEL	PAGE1
		call	disp_clr1
		call	disp_clr2
		PAGESEL	PAGE3

;start I2C communication
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'01000000'
		call	i2c_send
		PAGESEL	PAGE3

		movlw	B'11111110'
		call	demo8574_send
		movlw	B'11111101'
		call	demo8574_send
		movlw	B'11111011'
		call	demo8574_send
		movlw	B'11110111'
		call	demo8574_send
		movlw	B'11101111'
		call	demo8574_send
		movlw	B'11011111'
		call	demo8574_send
		movlw	B'10111111'
		call	demo8574_send
		movlw	B'01111111'
		call	demo8574_send
		movlw	B'11111111'
		call	demo8574_send

;finish the communication
		PAGESEL	PAGE0
		call	i2c_stop
		PAGESEL	PAGE3

		goto	p_75_3
;------------------------------------------------------------------------------------
demo8574_send	bsf	STATUS,RP1
		movwf	v_p0
		bcf	STATUS,RP1
		PAGESEL	PAGE2
		call	w_demo8574
		PAGESEL	PAGE0
		bsf	STATUS,RP1
		movf	v_p0,0
		bcf	STATUS,RP1
		call	i2c_send
		PAGESEL	PAGE3
		call	d1333
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------

p_key0		nop

		bsf	STATUS,RP1
;-- H -----------------------------------
		movf	v_cursor2,0
		xorlw	D'015'
		btfsc	STATUS,Z
		bcf	v_p1,D'007'
;
		movf	v_cursor2,0
		xorlw	D'014'
		btfsc	STATUS,Z
		bcf	v_p1,D'006'
;
		movf	v_cursor2,0
		xorlw	D'013'
		btfsc	STATUS,Z
		bcf	v_p1,D'005'
;
		movf	v_cursor2,0
		xorlw	D'012'
		btfsc	STATUS,Z
		bcf	v_p1,D'004'
;
		movf	v_cursor2,0
		xorlw	D'011'
		btfsc	STATUS,Z
		bcf	v_p1,D'003'
;
		movf	v_cursor2,0
		xorlw	D'010'
		btfsc	STATUS,Z
		bcf	v_p1,D'002'
;
		movf	v_cursor2,0
		xorlw	D'009'
		btfsc	STATUS,Z
		bcf	v_p1,D'001'
;
		movf	v_cursor2,0
		xorlw	D'008'
		btfsc	STATUS,Z
		bcf	v_p1,D'000'
;-- L -----------------------------------
		movf	v_cursor2,0
		xorlw	D'007'
		btfsc	STATUS,Z
		bcf	v_p0,D'007'
;
		movf	v_cursor2,0
		xorlw	D'006'
		btfsc	STATUS,Z
		bcf	v_p0,D'006'
;
		movf	v_cursor2,0
		xorlw	D'005'
		btfsc	STATUS,Z
		bcf	v_p0,D'005'
;
		movf	v_cursor2,0
		xorlw	D'004'
		btfsc	STATUS,Z
		bcf	v_p0,D'004'
;
		movf	v_cursor2,0
		xorlw	D'003'
		btfsc	STATUS,Z
		bcf	v_p0,D'003'
;
		movf	v_cursor2,0
		xorlw	D'002'
		btfsc	STATUS,Z
		bcf	v_p0,D'002'
;
		movf	v_cursor2,0
		xorlw	D'001'
		btfsc	STATUS,Z
		bcf	v_p0,D'001'
;
		movf	v_cursor2,0
		xorlw	D'000'
		btfsc	STATUS,Z
		bcf	v_p0,D'000'
;
		bcf	STATUS,RP1
		goto	inc_c_pos
;----------------------------------------------------------
;----------------------------------------------------------
p_key1		nop

		bsf	STATUS,RP1
;-- H -----------------------------------
		movf	v_cursor2,0
		xorlw	D'015'
		btfsc	STATUS,Z
		bsf	v_p1,D'007'
;
		movf	v_cursor2,0
		xorlw	D'014'
		btfsc	STATUS,Z
		bsf	v_p1,D'006'
;
		movf	v_cursor2,0
		xorlw	D'013'
		btfsc	STATUS,Z
		bsf	v_p1,D'005'
;
		movf	v_cursor2,0
		xorlw	D'012'
		btfsc	STATUS,Z
		bsf	v_p1,D'004'
;
		movf	v_cursor2,0
		xorlw	D'011'
		btfsc	STATUS,Z
		bsf	v_p1,D'003'
;
		movf	v_cursor2,0
		xorlw	D'010'
		btfsc	STATUS,Z
		bsf	v_p1,D'002'
;
		movf	v_cursor2,0
		xorlw	D'009'
		btfsc	STATUS,Z
		bsf	v_p1,D'001'
;
		movf	v_cursor2,0
		xorlw	D'008'
		btfsc	STATUS,Z
		bsf	v_p1,D'000'
;-- L -----------------------------------
		movf	v_cursor2,0
		xorlw	D'007'
		btfsc	STATUS,Z
		bsf	v_p0,D'007'
;
		movf	v_cursor2,0
		xorlw	D'006'
		btfsc	STATUS,Z
		bsf	v_p0,D'006'
;
		movf	v_cursor2,0
		xorlw	D'005'
		btfsc	STATUS,Z
		bsf	v_p0,D'005'
;
		movf	v_cursor2,0
		xorlw	D'004'
		btfsc	STATUS,Z
		bsf	v_p0,D'004'
;
		movf	v_cursor2,0
		xorlw	D'003'
		btfsc	STATUS,Z
		bsf	v_p0,D'003'
;
		movf	v_cursor2,0
		xorlw	D'002'
		btfsc	STATUS,Z
		bsf	v_p0,D'002'
;
		movf	v_cursor2,0
		xorlw	D'001'
		btfsc	STATUS,Z
		bsf	v_p0,D'001'
;
		movf	v_cursor2,0
		xorlw	D'000'
		btfsc	STATUS,Z
		bsf	v_p0,D'000'
;
		bcf	STATUS,RP1
		goto	inc_c_pos
;--------------------------------------------------
inc_c_pos	call	d332		;wait a while...

		bsf	STATUS,RP1
		decf	v_cursor2,1
		movf	v_cursor2,0
		xorlw	D'255'
		btfsc	STATUS,Z
		goto	set_c1
		bcf	STATUS,RP1
		goto	p_75_1

set_c1		movlw	D'015'		;PCF8575 (16-bit)
		movwf	v_cursor2

		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z
		goto	set_c2

		bcf	STATUS,RP1
		goto	p_75_1

set_c2		movlw	D'007'		;not PCF8575 (8-bit)
		movwf	v_cursor2

		bcf	STATUS,RP1
		goto	p_75_1

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------

;------------------------------------------------------------------------------------
;TPIC2810 data send routines
;------------------------------------------------------------------------------------
p_send010	bcf	STATUS,RP1

		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'11000000'		;TPIC2810 I2C address + write
		call	i2c_send
		movlw	H'44'			;subaddress= 44h
		call	i2c_send

		bsf	STATUS,RP1
		movf	v_p0,0			;data to be send
		bcf	STATUS,RP1

		call	i2c_send
		call	i2c_stop
		PAGESEL	PAGE3
		return

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
p_send9534A	bcf	STATUS,RP1
		movlw	B'01110000'
		movwf	v_i2c_add
		goto	p_send6_gen
;---------------------------------------------------------------------------------
p_send6		bcf	STATUS,RP1
		movlw	B'00110000'
		movwf	v_i2c_add
		goto	p_send6_gen
;---------------------------------------------------------------------------------

;------------------------------------------------------------------------------------
;PCA6107 data send routines
;------------------------------------------------------------------------------------
p_send6_gen	bcf	STATUS,RP1

		PAGESEL	PAGE0

;1 - set all 8 bits to output ------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000011'		;pointer to config register
		call	i2c_send
		movlw	B'00000000'		;all bits as output
		call	i2c_send
		call	i2c_stop

;2 - write v_p0 to the device ------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000001'		;pointer to output register
		call	i2c_send
		bsf	STATUS,RP1
		movf	v_p0,0			;data to be send
		bcf	STATUS,RP1
		call	i2c_send
		call	i2c_stop
		PAGESEL	PAGE3
		return

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;general entry for I2C send routines for Texas Instruments Expanders
;data send routines
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_send1		nop

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'003'		;PCA6107
		btfsc	STATUS,Z
		goto	p_send6

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'004'		;PCA9543A
		btfsc	STATUS,Z
		goto	p_send9534A

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'005'		;MCP23008
		btfsc	STATUS,Z
		goto	p_send23008_c

		movf	v_p_device,0
		xorlw	D'010'		;TPIC2810
		btfsc	STATUS,Z
		goto	p_send010

		bcf	STATUS,RP1

;------------------------------------------------------------------------------------
;PCF8574 / PCF8575 data send routines
;------------------------------------------------------------------------------------
		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'01000000'
		call	i2c_send

		bsf	STATUS,RP1
		movf	v_p0,0		;first transmitted byte is LSB (P0x)
		bcf	STATUS,RP1
		call	i2c_send

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z	;continue (send second byte) only for PCF8575
		goto	ps1_1		;stop communication for PCF8574(A)

		bsf	STATUS,RP1
		movf	v_p1,0		;second transmitted byte is LSB (P1x)
		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_send

ps1_1		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_stop

		PAGESEL	PAGE3
		return

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------



p_send23008_c	nop
		PAGESEL	PAGE2
		goto	p_send23008


p_read23008_c	nop
		PAGESEL	PAGE2
		goto	p_read23008



;------------------------------------------------------------------------------------
;TPIC2810 data read routines
;------------------------------------------------------------------------------------
p_read010	bcf	STATUS,RP1

		PAGESEL	PAGE0

;1 - set all 8 bits to input ------------
		call	i2c_start
		movlw	B'11000000'		;TPIC2810 I2C address + write
		call	i2c_send
		movlw	H'11'			;subaddress
		call	i2c_send

		call	i2c_restart
		movlw	B'11000001'		;TPIC2810 I2C address + read
		call	i2c_send
		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r0
		bcf	STATUS,RP1

		call	i2c_not_ack
		call	i2c_stop
		PAGESEL	PAGE3
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read6		bcf	STATUS,RP1
		movlw	B'00110000'		;PCA6107
		movwf	v_i2c_add
		goto	p_read6_gen
;------------------------------------------------------------------------------------
p_read9534A	bcf	STATUS,RP1
		movlw	B'01110000'		;PCA9534A
		movwf	v_i2c_add
		goto	p_read6_gen

;------------------------------------------------------------------------------------


;------------------------------------------------------------------------------------
;PCA6107 data read routines
;------------------------------------------------------------------------------------
p_read6_gen	bcf	STATUS,RP1

		PAGESEL	PAGE0

;1 - set all 8 bits to input ------------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000011'		;pointer to config register
		call	i2c_send
		movlw	B'11111111'		;all bits as input
		call	i2c_send
		call	i2c_stop

;2 - set inversion register ------------
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000010'		;pointer to polarity inversion register
		call	i2c_send
		movlw	B'00000000'		;all bits w/o polarity inversion
		call	i2c_send
		call	i2c_stop

;3 - receive data from the device ------
;    and write to v_r0
		call	i2c_start
		movf	v_i2c_add,0		;device I2C address + write
		call	i2c_send
		movlw	B'00000000'		;pointer to input register
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movf	v_i2c_add,0		;device I2C address
		addlw	D'001'			;+ read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r0
		bcf	STATUS,RP1

		call	i2c_not_ack
		call	i2c_stop
		PAGESEL	PAGE3
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;general entry for I2C read routines for Texas Instruments Expanders
;data receive routines
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
p_read1		nop

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'003'		;PCA6107
		btfsc	STATUS,Z
		goto	p_read6

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'004'		;PCA9534A
		btfsc	STATUS,Z
		goto	p_read9534A

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'005'		;MCP23008
		btfsc	STATUS,Z
		goto	p_read23008_c

		movf	v_p_device,0
		xorlw	D'010'		;TPIC2810
		btfsc	STATUS,Z
		goto	p_read010

		bcf	STATUS,RP1

;------------------------------------------------------------------------------------
;PCF8574 / PCF8575 data send routines
;------------------------------------------------------------------------------------
		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'01000001'
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r0		;first transmitted byte is LSB (P0x)

		PAGESEL	PAGE3
		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z	;continue (ACK + receive second byte) only for PCF8575
		goto	pr1_1		;stop communication for PCF8574(A)

		bcf	STATUS,RP1

		PAGESEL	PAGE0
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r1		;second transmitted byte is LSB (P1x)
pr1_1		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; NXP devices (port) I2C communication routines
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

p9530_i2c_read	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

;1. set outputs to high impedance
		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movlw	B'00010101'
		movwf	v_F0
		movf	v_pca953x,0
		xorlw	D'003'
		btfsc	STATUS,Z
		incf	v_F0,1			;LS0 starts at other address for PCA9532

		movf	v_F0,0			;select LS0 register + autoincrement
		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_send

		bsf	STATUS,RP1
		movlw	H'F0'
		movwf	v_F0
		movf	v_pca953x,0
		xorlw	D'001'
		btfss	STATUS,Z
		clrf	v_F0			;for all devices except of PCA9530
		movf	v_F0,0
		bcf	STATUS,RP1
		call	i2c_send		;set LS1 to 00000000 (or 11110000 for PCA9530)

;for PCA9531 and PCA9532 we have to continue
		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x789			;leave for PCA9530 now
		bcf	STATUS,RP1

		PAGESEL	PAGE0
		movlw	H'00'
		call	i2c_send		;set LS1 to 00000000
		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x789

;for PCA9532 set registers LS2 & LS3 to 00000000
		bcf	STATUS,RP1
		PAGESEL	PAGE0
		movlw	H'00'
		call	i2c_send
		movlw	H'00'
		call	i2c_send

x789		bcf	STATUS,RP1
		PAGESEL	PAGE0

		call	i2c_stop

;2. read the input value from INPUT0
		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		movlw	B'00000000'		;select INPUT register
		call	i2c_send

		call	i2c_start
		movlw	B'11000001'		;device I2C address + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r0
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		bcf	STATUS,RP1
		xorlw	D'003'
		btfss	STATUS,Z
		return

;read the INPUT2 register for device PCA9532
		nop
		PAGESEL	PAGE0

		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		movlw	B'00000001'		;select INPUT1 (2nd) register
		call	i2c_send

		call	i2c_start
		movlw	B'11000001'		;device I2C address + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_r1
		bcf	STATUS,RP1
		call	i2c_not_ack
		call	i2c_stop

		PAGESEL	PAGE3
		return

;-------------------------------------------------------------------------------
; PCA9530 - all registers read (I2C):
; PWM, RSC and LS0
;-------------------------------------------------------------------------------
p_pxx_i2c_read	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		bcf	STATUS,RP1
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	x593
;-----------------------------------------------
;PCA9530 & PCA9531
		nop
		PAGESEL	PAGE0
		movlw	B'00010001'		;select PSC0 register + autoincrement
		call	i2c_send
		PAGESEL	PAGE3
		goto	x594
;-----------------------------------------------
;PCA9532
x593		nop
		PAGESEL	PAGE0
		movlw	B'00010010'		;select PSC0 register + autoincrement
		call	i2c_send
;-----------------------------------------------
x594		nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'11000001'		;device I2C address + read
		call	i2c_send

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_psc0
		bcf	STATUS,RP1
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_pwm0
		bcf	STATUS,RP1
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_psc1
		bcf	STATUS,RP1
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_pwm1
		bcf	STATUS,RP1
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_ls0
		bcf	STATUS,RP1
		call	i2c_ack

;we do not need to leave for PCA9530 now
;we just start to read something what doesn't make sense anymore
;but doesn't hurt as well....

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_ls1
		bcf	STATUS,RP1
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_ls2
		bcf	STATUS,RP1
		call	i2c_ack

		call	i2c_receive
		bsf	STATUS,RP1
		movwf	v_ls3
		bcf	STATUS,RP1
		call	i2c_not_ack

		call	i2c_stop
		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------------
; PWM & PSC registers I2C write
;-------------------------------------------------------------------------------
p_pxx_i2c_write	nop
		PAGESEL	PAGE0
		call	i2c_start
		movlw	B'11000000'		;device I2C address + write
		call	i2c_send

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	x578
;--------------------------------------------------------		
;PCA9530 & PCA9531
		bcf	STATUS,RP1
		PAGESEL	PAGE0
		movlw	B'00010001'		;select PSC0 register + autoincrement
		call	i2c_send
		PAGESEL	PAGE3
		goto	x577
;--------------------------------------------------------
;PCA9532 - PSC0 register starts at 0010 instead of 0001
x578		bcf	STATUS,RP1
		PAGESEL	PAGE0
		movlw	B'00010010'		;select PSC0 register + autoincrement
		call	i2c_send
;--------------------------------------------------------
x577		bsf	STATUS,RP1
		PAGESEL	PAGE0
		movf	v_psc0,0
		bcf	STATUS,RP1
		call	i2c_send

		bsf	STATUS,RP1
		movf	v_pwm0,0
		bcf	STATUS,RP1
		call	i2c_send

		bsf	STATUS,RP1
		movf	v_psc1,0
		bcf	STATUS,RP1
		call	i2c_send

		bsf	STATUS,RP1
		movf	v_pwm1,0
		bcf	STATUS,RP1
		call	i2c_send

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'001'
		btfss	STATUS,Z
		goto	next22
;only for PCA9530 - set the first highest bits to 1
		movlw	B'11110000'
		iorwf	v_ls0,1
next22		nop
		PAGESEL	PAGE0
		movf	v_ls0,0
		bcf	STATUS,RP1
		call	i2c_send

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x777

;send LS1 for devices above PCA9530
		movf	v_ls1,0
		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_send

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x777

;in addition send LS2 and LS3 for PCA9532
		movf	v_ls2,0
		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_send

		bsf	STATUS,RP1
		movf	v_ls3,0
		bcf	STATUS,RP1
		call	i2c_send

x777		bcf	STATUS,RP1
		PAGESEL	PAGE0
		call	i2c_stop
		PAGESEL	PAGE3
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


w_ports_m	nop
		PAGESEL	PAGE2
		goto	w_ports_m_r

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_ppwm		nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_S
		call	wm_C
		call	wm_0
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		bsf	STATUS,RP1
		movf	v_psc0,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_minus
		movlw	'>'
		call	send_data
		call	wm_space

		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_0
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		bsf	STATUS,RP1
		movf	v_pwm0,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_S
		call	wm_C
		call	wm_1
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		bsf	STATUS,RP1
		movf	v_psc1,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		call	wm_space
		call	wm_minus
		movlw	'>'
		call	send_data
		call	wm_space

		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_1
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		bsf	STATUS,RP1
		movf	v_pwm1,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		call	write_hex
		call	wm_H

		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_pca9530m	nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_C
		call	wm_A
		call	wm_9
		call	wm_5
		call	wm_3

;------------------------------------
		bsf	STATUS,RP1
		movf	v_pca953x,0
		bcf	STATUS,RP1
		xorlw	D'001'
		btfsc	STATUS,Z
		call	wm_0

		bsf	STATUS,RP1
		movf	v_pca953x,0
		bcf	STATUS,RP1
		xorlw	D'002'
		btfsc	STATUS,Z
		call	wm_1
;------------------------------------

		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_A
		call	wm_dvojbodka
		call	wm_1
		call	wm_1
		call	wm_0
		call	wm_0
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0

		movlw	B'11000000'
		call	send_comm
		call	wm_zat1
		call	wm_R
		call	wm_zat2
		call	wm_e
		call	wm_a
		call	wm_d

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_zat1
		call	wm_P
		call	wm_zat2
		call	wm_W
		call	wm_M

		call	wm_space
		call	wm_sls
		call	wm_space

		call	wm_zat1
		call	wm_L
		call	wm_zat2
		call	wm_E
		call	wm_D
		call	wm_s

		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
w_pca9530r	nop
		PAGESEL	PAGE1
		movlw	B'11000000'
		call	send_comm

		movlw	'<'
		call	send_data

		bsf	STATUS,RP1
		movf	v_r0,0
		bcf	STATUS,RP1
		movwf	c_tmp

		PAGESEL	PAGE3
		call	w_p_data_s
		PAGESEL	PAGE1

		movlw	'>'
		call	send_data
		PAGESEL	PAGE3

		bsf	STATUS,RP1
		movf	v_pca953x,0
		bcf	STATUS,RP1
		xorlw	D'003'
		btfss	STATUS,Z
		return

;for PCA9532 display the second input register
		nop
		PAGESEL	PAGE1
		call	wm_minus
		movlw	'<'
		call	send_data

		bsf	STATUS,RP1
		movf	v_r1,0
		bcf	STATUS,RP1
		movwf	c_tmp

		PAGESEL	PAGE3
		call	w_p_data_s
		PAGESEL	PAGE1

		movlw	'>'
		call	send_data


		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
w_LED0		nop
		PAGESEL	PAGE1
		movlw	B'10000000'
		call	send_comm

		call	wm_L
		call	wm_S
		call	wm_x
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space
		bsf	STATUS,RP1
		movf	v_ls0,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	x977
;not PCA9530
		bcf	STATUS,RP1
		PAGESEL	PAGE1
		call	wm_minus
		bsf	STATUS,RP1
		movf	v_ls1,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'003'
		btfss	STATUS,Z
		goto	x977
;PCA9532
		nop
		bcf	STATUS,RP1
		PAGESEL	PAGE1
		call	wm_minus
		bsf	STATUS,RP1
		movf	v_ls2,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

		bcf	STATUS,RP1
		call	wm_minus
		bsf	STATUS,RP1
		movf	v_ls3,0
		bcf	STATUS,RP1
		movwf	kbd_buffer
		movwf	c_tmp
		call	write_hex
		call	wm_H

;-------------------------------------------------
x977		nop
;second line - LED status ------------------------
		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_pca953x,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	w_LED0_1
		bcf	STATUS,RP1
;PCA9531 & PCA9532 part --------------------------
		PAGESEL	PAGE1
		movlw	B'11000000'
		call	send_comm
		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_dvojbodka
		call	wm_space

		PAGESEL	PAGE3
		bsf	STATUS,RP1
		movf	v_ls0,0
		movwf	v_lsx
		bcf	STATUS,RP1
		call	w_LED_st4

		bsf	STATUS,RP1
		movf	v_ls1,0
		movwf	v_lsx
		bcf	STATUS,RP1
		call	w_LED_st4
;PCA9532 part ------------------------------------
		bsf	STATUS,RP1
		movf	v_pca953x,0
		bcf	STATUS,RP1
		xorlw	D'003'
		btfss	STATUS,Z
		return				;leave now for PCA9531

		bsf	STATUS,RP1
		movf	v_ls2,0
		movwf	v_lsx
		bcf	STATUS,RP1
		call	w_LED_st4

		bsf	STATUS,RP1
		movf	v_ls3,0
		movwf	v_lsx
		bcf	STATUS,RP1
		call	w_LED_st4
		return
;PCA9530 part ------------------------------------
w_LED0_1	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'11000000'
		call	send_comm
		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_0
		call	wm_dvojbodka
		call	wm_space

		PAGESEL	PAGE3
		call	w_LEDx
		PAGESEL	PAGE1

		movlw	B'11001011'
		call	send_comm
		movlw	'*'
		call	send_data
		call	wm_space
		call	wm_L
		call	wm_E
		call	wm_D
		call	wm_1
		call	wm_dvojbodka
		call	wm_space

		bsf	STATUS,RP1
		movf	v_ls0,0
		bcf	STATUS,RP1
		movwf	c_tmp
		rrf	c_tmp,1
		rrf	c_tmp,1
		PAGESEL	PAGE3
		call	w_LEDx

		return
;-------------------------------------------------------------------------
w_LED_st4	bsf	STATUS,RP1
		movf	v_lsx,0
		bcf	STATUS,RP1
		movwf	c_tmp
		call	w_LEDx2

		bsf	STATUS,RP1
		movf	v_lsx,0
		bcf	STATUS,RP1
		movwf	c_tmp
		rrf	c_tmp,1
		rrf	c_tmp,1
		call	w_LEDx2

		bsf	STATUS,RP1
		movf	v_lsx,0
		bcf	STATUS,RP1
		movwf	c_tmp
		rrf	c_tmp,1
		rrf	c_tmp,1
		rrf	c_tmp,1
		rrf	c_tmp,1
		call	w_LEDx2

		bsf	STATUS,RP1
		movf	v_lsx,0
		bcf	STATUS,RP1
		movwf	c_tmp
		rrf	c_tmp,1
		rrf	c_tmp,1
		rrf	c_tmp,1
		rrf	c_tmp,1
		rrf	c_tmp,1
		rrf	c_tmp,1
		call	w_LEDx2
		PAGESEL	PAGE1
		call	wm_space
		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
w_LEDx2		nop
		PAGESEL	PAGE1
		movlw	B'00000011'
		andwf	c_tmp,1
		movf	c_tmp,0
		btfsc	STATUS,Z
		call	wm_0
		movf	c_tmp,0
		xorlw	B'00000001'
		btfsc	STATUS,Z
		call	wm_1
		movf	c_tmp,0
		xorlw	B'00000010'
		btfsc	STATUS,Z
		call	wm_2
		movf	c_tmp,0
		xorlw	B'00000011'
		btfsc	STATUS,Z
		call	wm_3
		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
w_LEDx		movlw	B'00000011'
		andwf	c_tmp,1
		movf	c_tmp,0
		btfsc	STATUS,Z
		call	w_LED_off
		movf	c_tmp,0
		xorlw	B'00000001'
		btfsc	STATUS,Z
		call	w_LED_on
		movf	c_tmp,0
		xorlw	B'00000010'
		btfsc	STATUS,Z
		call	w_LED_PWM0
		movf	c_tmp,0
		xorlw	B'00000011'
		btfsc	STATUS,Z
		call	w_LED_PWM1
		return
;----------------------------------------------------
w_LED_off	nop
		PAGESEL	PAGE1
		call	wm_O
		call	wm_f
		call	wm_f
		call	wm_space
		PAGESEL	PAGE3
		return
;----------------------------------------------------
w_LED_on	nop
		PAGESEL	PAGE1
		call	wm_O
		call	wm_n
		call	wm_space
		call	wm_space
		PAGESEL	PAGE3
		return
;----------------------------------------------------
w_LED_PWM0	nop
		PAGESEL	PAGE1
		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_0
		PAGESEL	PAGE3
		return
;----------------------------------------------------
w_LED_PWM1	nop
		PAGESEL	PAGE1
		call	wm_P
		call	wm_W
		call	wm_M
		call	wm_1
		PAGESEL	PAGE3
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_pca6107g	nop
		PAGESEL	PAGE2
		goto	w_pca6107
;-------------------------------------------------------------------------
w_mcp23008g	nop
		PAGESEL	PAGE2
		goto	w_mcp23008
;-------------------------------------------------------------------------
w_pca9534Ag	nop
		PAGESEL	PAGE2
		goto	w_pca9534A
;-------------------------------------------------------------------------
w_pcf8574g	nop
		PAGESEL	PAGE2
		goto	w_pcf8574
;-------------------------------------------------------------------------
w_tpic2810g	nop
		PAGESEL	PAGE2
		goto	w_tpic2810
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_pcf8575	nop
		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	w_pcf8574g

		movf	v_p_device,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	w_pca6107g

		movf	v_p_device,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	w_pca9534Ag

		movf	v_p_device,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	w_mcp23008g

		movf	v_p_device,0
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	w_tpic2810g
		bcf	STATUS,RP1

		PAGESEL	PAGE1

		movlw	B'10001100'		;1/12
		call	send_comm

		call	wm_minus
		movlw	'<'
		call	send_data
		movlw	B'10010111'		;1/23
		call	send_comm
		movlw	'>'
		call	send_data

w_p0_main	bcf	STATUS,RP1
		PAGESEL	PAGE1
		movlw	B'11001100'		;2/12
		call	send_comm

		call	wm_minus
		movlw	'<'
		call	send_data
		movlw	B'11010111'		;2/23
		call	send_comm
		movlw	'>'
		call	send_data


		PAGESEL	PAGE3

		call	w_p_data
		call	w_r_data

;---------------------------------------------
;set the cursor for "binary editor"
		call	set_cursor2
		bcf	STATUS,RP1

		PAGESEL	PAGE1
		call	send_comm
		PAGESEL	PAGE3
;---------------------------------------------

		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_r_data	nop
		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z
		goto	w_rd2
		bcf	STATUS,RP1

		PAGESEL	PAGE1

		movlw	B'10001110'		;1/14
		call	send_comm


		bsf	STATUS,RP1
		movf	v_r1,0
		bcf	STATUS,RP1
		movwf	c_tmp

		PAGESEL	PAGE3
		call	w_p_data_s
		PAGESEL	PAGE1


w_rd2		bcf	STATUS,RP1
		PAGESEL	PAGE1

		movlw	B'11001110'		;2/14
		call	send_comm

		bsf	STATUS,RP1
		movf	v_r0,0
		bcf	STATUS,RP1
		movwf	c_tmp

		PAGESEL	PAGE3
		call	w_p_data_s


		return

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_p_data	nop

		bsf	STATUS,RP1
		movf	v_p_device,0
		xorlw	D'002'
		btfss	STATUS,Z
		goto	w_p2
		bcf	STATUS,RP1

		PAGESEL	PAGE1

		movlw	B'10000000'		;1/0
		call	send_comm

		call	wm_P
		call	wm_1
		call	wm_dvojbodka

		bsf	STATUS,RP1
		movf	v_p1,0
		bcf	STATUS,RP1
		movwf	c_tmp

		PAGESEL	PAGE3
		call	w_p_data_s
		PAGESEL	PAGE1


w_p2		bcf	STATUS,RP1
		PAGESEL	PAGE1

		movlw	B'11000000'		;2/0
		call	send_comm

		call	wm_P
		call	wm_0
		call	wm_dvojbodka

		bsf	STATUS,RP1
		movf	v_p0,0
		bcf	STATUS,RP1
		movwf	c_tmp

		PAGESEL	PAGE3
		call	w_p_data_s

		return
		
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
w_p_data_s	nop
		PAGESEL	PAGE1
		movlw	'1'
		btfss	c_tmp,D'007'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'006'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'005'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'004'
		movlw	'0'
		call	send_data

		call	wm_space

		movlw	'1'
		btfss	c_tmp,D'003'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'002'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'001'
		movlw	'0'
		call	send_data

		movlw	'1'
		btfss	c_tmp,D'000'
		movlw	'0'
		call	send_data

		PAGESEL	PAGE3
		return

;------------------------------------------------------
set_cursor2	bsf	STATUS,RP1
		movf	v_cursor2,0
		xorlw	D'015'
		btfsc	STATUS,Z
		goto	wpdsc_15
;
		movf	v_cursor2,0
		xorlw	D'014'
		btfsc	STATUS,Z
		goto	wpdsc_14
;
		movf	v_cursor2,0
		xorlw	D'013'
		btfsc	STATUS,Z
		goto	wpdsc_13
;
		movf	v_cursor2,0
		xorlw	D'012'
		btfsc	STATUS,Z
		goto	wpdsc_12
;
		movf	v_cursor2,0
		xorlw	D'011'
		btfsc	STATUS,Z
		goto	wpdsc_11
;
		movf	v_cursor2,0
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	wpdsc_10
;
		movf	v_cursor2,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	wpdsc_09
;
		movf	v_cursor2,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	wpdsc_08
;
		movf	v_cursor2,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	wpdsc_07
;
		movf	v_cursor2,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	wpdsc_06
;
		movf	v_cursor2,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	wpdsc_05
;
		movf	v_cursor2,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	wpdsc_04
;
		movf	v_cursor2,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	wpdsc_03
;
		movf	v_cursor2,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	wpdsc_02
;
		movf	v_cursor2,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	wpdsc_01
;------------------------------------------
		movlw	B'11001011'	;11
		return
;------------------------------------------
wpdsc_01	movlw	B'11001010'	;10
		return
;------------------------------------------
wpdsc_02	movlw	B'11001001'	;09
		return
;------------------------------------------
wpdsc_03	movlw	B'11001000'	;08
		return
;------------------------------------------
wpdsc_04	movlw	B'11000110'	;06
		return
;------------------------------------------
wpdsc_05	movlw	B'11000101'	;05
		return
;------------------------------------------
wpdsc_06	movlw	B'11000100'	;04
		return
;------------------------------------------
wpdsc_07	movlw	B'11000011'	;03
		return
;------------------------------------------
wpdsc_08	movlw	B'10001011'	;11
		return
;------------------------------------------
wpdsc_09	movlw	B'10001010'	;10
		return
;------------------------------------------
wpdsc_10	movlw	B'10001001'	;09
		return
;------------------------------------------
wpdsc_11	movlw	B'10001000'	;08
		return
;------------------------------------------
wpdsc_12	movlw	B'10000110'	;06
		return
;------------------------------------------
wpdsc_13	movlw	B'10000101'	;05
		return
;------------------------------------------
wpdsc_14	movlw	B'10000100'	;04
		return
;------------------------------------------
wpdsc_15	movlw	B'10000011'	;03
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

d332		MOVLW   D'001'
		MOVWF   TIMER2
DEL_LOOP331	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP332	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP332
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP331
		return
;---------------------------------------------
d1332		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1331	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP1332	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP1332
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1331
		return
;---------------------------------------------
d1333		call	d1332
		call	d1332
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END